@@ -28,6 +28,7 @@
 #include "TelepathyQt4/debug-internal.h"
 
 #include <TelepathyQt4/Account>
+#include <TelepathyQt4/AccountFactory>
 #include <TelepathyQt4/PendingFailure>
 #include <TelepathyQt4/PendingReady>
 #include <TelepathyQt4/PendingVoid>
@@ -39,7 +40,7 @@ namespace Tp
 
 struct TELEPATHY_QT4_NO_EXPORT ChannelRequest::Private
 {
-    Private(ChannelRequest *parent);
+    Private(ChannelRequest *parent, const QVariantMap &immutableProperties);
     ~Private();
 
     static void introspectMain(Private *self);
@@ -49,9 +50,17 @@ struct TELEPATHY_QT4_NO_EXPORT ChannelRequest::Private
     // Public object
     ChannelRequest *parent;
 
+    // Context
+    AccountFactoryConstPtr accFact;
+    ConnectionFactoryConstPtr connFact;
+    ChannelFactoryConstPtr chanFact;
+    ContactFactoryConstPtr contactFact;
+
     // Instance of generated interface class
     Client::ChannelRequestInterface *baseInterface;
 
+    QVariantMap immutableProperties;
+
     // Optional interface proxies
     Client::DBus::PropertiesInterface *properties;
 
@@ -64,11 +73,13 @@ struct TELEPATHY_QT4_NO_EXPORT ChannelRequest::Private
     QualifiedPropertyValueMapList requests;
 };
 
-ChannelRequest::Private::Private(ChannelRequest *parent)
+ChannelRequest::Private::Private(ChannelRequest *parent,
+        const QVariantMap &immutableProperties)
     : parent(parent),
       baseInterface(new Client::ChannelRequestInterface(
                     parent->dbusConnection(), parent->busName(),
                     parent->objectPath(), parent)),
+      immutableProperties(immutableProperties),
       properties(0),
       readinessHelper(parent->readinessHelper())
 {
@@ -106,33 +117,89 @@ void ChannelRequest::Private::introspectMain(ChannelRequest::Private *self)
         Q_ASSERT(self->properties != 0);
     }
 
-    debug() << "Calling Properties::GetAll(ChannelRequest)";
-    QDBusPendingCallWatcher *watcher =
-        new QDBusPendingCallWatcher(
-                self->properties->GetAll(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_REQUEST)),
-                self->parent);
-    // FIXME: This is a Qt bug fixed upstream, should be in the next Qt release.
-    //        We should not need to check watcher->isFinished() here, remove the
-    //        check when a fixed Qt version is released.
-    if (watcher->isFinished()) {
-        self->parent->gotMainProperties(watcher);
+    QVariantMap props;
+    QString key;
+    bool needIntrospectMainProps = false;
+    const char *propertiesNames[] = { "Account", "UserActionTime",
+        "PreferredHandler", "Requests", "Interfaces",
+        NULL };
+    for (unsigned i = 0; propertiesNames[i] != NULL; ++i) {
+        key = QLatin1String(TELEPATHY_INTERFACE_CHANNEL_REQUEST ".");
+        key += QLatin1String(propertiesNames[i]);
+        if (!self->immutableProperties.contains(key)) {
+            needIntrospectMainProps = true;
+            break;
+        }
+        props.insert(QLatin1String(propertiesNames[i]),
+                self->immutableProperties[key]);
+    }
+
+    if (needIntrospectMainProps) {
+        debug() << "Calling Properties::GetAll(ChannelRequest)";
+        QDBusPendingCallWatcher *watcher =
+            new QDBusPendingCallWatcher(
+                    self->properties->GetAll(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_REQUEST)),
+                    self->parent);
+        // FIXME: This is a Qt bug fixed upstream, should be in the next Qt release.
+        //        We should not need to check watcher->isFinished() here, remove the
+        //        check when a fixed Qt version is released.
+        if (watcher->isFinished()) {
+            self->parent->gotMainProperties(watcher);
+        } else {
+            self->parent->connect(watcher,
+                    SIGNAL(finished(QDBusPendingCallWatcher*)),
+                    SLOT(gotMainProperties(QDBusPendingCallWatcher*)));
+        }
     } else {
-        self->parent->connect(watcher,
-                SIGNAL(finished(QDBusPendingCallWatcher*)),
-                SLOT(gotMainProperties(QDBusPendingCallWatcher*)));
+        self->extractMainProps(props);
     }
 }
 
 void ChannelRequest::Private::extractMainProps(const QVariantMap &props)
 {
-    parent->setInterfaces(qdbus_cast<QStringList>(props.value(QLatin1String("Interfaces"))));
+    PendingReady *readyOp = 0;
 
-    if (!account && props.contains(QLatin1String("Account"))) {
+    if (props.contains(QLatin1String("Account"))) {
         QDBusObjectPath accountObjectPath =
             qdbus_cast<QDBusObjectPath>(props.value(QLatin1String("Account")));
-        account = Account::create(
-                QLatin1String(TELEPATHY_ACCOUNT_MANAGER_BUS_NAME),
-                accountObjectPath.path());
+
+        if (!account.isNull()) {
+            if (accountObjectPath.path() == account->objectPath()) {
+                // Most often a no-op, but we want this to guarantee the old behavior in all cases
+                readyOp = account->becomeReady();
+            } else {
+                warning() << "The account" << accountObjectPath.path() << "was not the expected"
+                    << account->objectPath() << "for CR" << parent->objectPath();
+                // Construct a new one instead
+                account.reset();
+            }
+        }
+
+        // We need to check again because we might have dropped the expected account just a sec ago
+        // Note that many of the if paths will go away in the API/ABI break - they're just for
+        // backwards compat
+        if (account.isNull()) {
+            if (!accFact.isNull()) {
+                readyOp = accFact->proxy(
+                        QLatin1String(TELEPATHY_ACCOUNT_MANAGER_BUS_NAME), accountObjectPath.path(),
+                        connFact, chanFact, contactFact);
+                account = AccountPtr::dynamicCast(readyOp->proxy());
+            } else {
+                // We might have the conn fact from the expected account, let's use that for good
+                // measure even if the account didn't match
+                if (!connFact.isNull()) {
+                    account = Account::create(
+                            QLatin1String(TELEPATHY_ACCOUNT_MANAGER_BUS_NAME),
+                            accountObjectPath.path(), connFact, chanFact, contactFact);
+                } else {
+                    account = Account::create(
+                            QLatin1String(TELEPATHY_ACCOUNT_MANAGER_BUS_NAME),
+                            accountObjectPath.path());
+                }
+
+                readyOp = account->becomeReady();
+            }
+        }
     }
 
     // FIXME See http://bugs.freedesktop.org/show_bug.cgi?id=21690
@@ -143,6 +210,18 @@ void ChannelRequest::Private::extractMainProps(const QVariantMap &props)
 
     preferredHandler = qdbus_cast<QString>(props.value(QLatin1String("PreferredHandler")));
     requests = qdbus_cast<QualifiedPropertyValueMapList>(props.value(QLatin1String("Requests")));
+
+    parent->setInterfaces(qdbus_cast<QStringList>(props[QLatin1String("Interfaces")]));
+    readinessHelper->setInterfaces(parent->interfaces());
+
+    if (account) {
+        parent->connect(readyOp,
+                SIGNAL(finished(Tp::PendingOperation *)),
+                SLOT(onAccountReady(Tp::PendingOperation *)));
+    } else {
+        warning() << "ChannelRequest.Account is missing or empty. Ignoring";
+        readinessHelper->setIntrospectCompleted(FeatureCore, true);
+    }
 }
 
 /**
@@ -204,6 +283,44 @@ ChannelRequestPtr ChannelRequest::create(const QDBusConnection &bus,
                 immutableProperties));
 }
 
+/**
+ * Create a new channel request object using the given \a bus and the given factories.
+ *
+ * \param objectPath The channel request object path.
+ * \param immutableProperties The immutable properties of the channel request.
+ * \param accountFactory The account factory to use.
+ * \param connectionFactory The connection factory to use.
+ * \param channelFactory The channel factory to use.
+ * \param contactFactory The contact factory to use.
+ * \return A ChannelRequestPtr pointing to the newly created ChannelRequest.
+ */
+ChannelRequestPtr ChannelRequest::create(const QDBusConnection &bus, const QString &objectPath,
+        const QVariantMap &immutableProperties,
+        const AccountFactoryConstPtr &accountFactory,
+        const ConnectionFactoryConstPtr &connectionFactory,
+        const ChannelFactoryConstPtr &channelFactory,
+        const ContactFactoryConstPtr &contactFactory)
+{
+    return ChannelRequestPtr(new ChannelRequest(bus, objectPath, immutableProperties,
+                accountFactory, connectionFactory, channelFactory, contactFactory));
+}
+
+/**
+ * Create a new channel request object for the given \a account.
+ *
+ * The returned instance will use factories from the account.
+ *
+ * \param account The account that the request was made through.
+ * \param objectPath The channel request object path.
+ * \param immutableProperties The immutable properties of the channel request.
+ * \return A ChannelRequestPtr pointing to the newly created ChannelRequest.
+ */
+ChannelRequestPtr ChannelRequest::create(const AccountPtr &account, const QString &objectPath,
+        const QVariantMap &immutableProperties)
+{
+    return ChannelRequestPtr(new ChannelRequest(account, objectPath, immutableProperties));
+}
+
 /**
  * Construct a new channel request object using the given \a bus.
  *
@@ -219,9 +336,76 @@ ChannelRequest::ChannelRequest(const QDBusConnection &bus,
             objectPath),
       OptionalInterfaceFactory<ChannelRequest>(this),
       ReadyObject(this, FeatureCore),
-      mPriv(new Private(this))
+      mPriv(new Private(this, immutableProperties))
 {
-    mPriv->extractMainProps(immutableProperties);
+}
+
+/**
+ * Construct a new channel request object using the given \a bus and the given factories.
+ *
+ * \param bus QDBusConnection to use.
+ * \param objectPath The channel request object path.
+ * \param accountFactory The account factory to use.
+ * \param connectionFactory The connection factory to use.
+ * \param channelFactory The channel factory to use.
+ * \param contactFactory The contact factory to use.
+ * \param immutableProperties The immutable properties of the channel request.
+ * \return A ChannelRequestPtr pointing to the newly created ChannelRequest.
+ */
+ChannelRequest::ChannelRequest(const QDBusConnection &bus,
+        const QString &objectPath, const QVariantMap &immutableProperties,
+        const AccountFactoryConstPtr &accountFactory,
+        const ConnectionFactoryConstPtr &connectionFactory,
+        const ChannelFactoryConstPtr &channelFactory,
+        const ContactFactoryConstPtr &contactFactory)
+    : StatefulDBusProxy(bus,
+            QLatin1String(TELEPATHY_INTERFACE_CHANNEL_DISPATCHER),
+            objectPath),
+      OptionalInterfaceFactory<ChannelRequest>(this),
+      ReadyObject(this, FeatureCore),
+      mPriv(new Private(this, immutableProperties))
+{
+    if (accountFactory->dbusConnection().name() != bus.name()) {
+        warning() << "  The D-Bus connection in the account factory is not the proxy connection";
+    }
+
+    if (connectionFactory->dbusConnection().name() != bus.name()) {
+        warning() << "  The D-Bus connection in the connection factory is not the proxy connection";
+    }
+
+    if (channelFactory->dbusConnection().name() != bus.name()) {
+        warning() << "  The D-Bus connection in the channel factory is not the proxy connection";
+    }
+
+    mPriv->accFact = accountFactory;
+    mPriv->connFact = connectionFactory;
+    mPriv->chanFact = channelFactory;
+    mPriv->contactFact = contactFactory;
+}
+
+/**
+ * Construct a new channel request object using the given \a account.
+ *
+ * The constructed instance will use the factories from the account.
+ *
+ * \param account Account to use.
+ * \param objectPath The channel request object path.
+ * \param immutableProperties The immutable properties of the channel request.
+ * \return A ChannelRequestPtr pointing to the newly created ChannelRequest.
+ */
+ChannelRequest::ChannelRequest(const AccountPtr &account,
+        const QString &objectPath, const QVariantMap &immutableProperties)
+    : StatefulDBusProxy(account->dbusConnection(),
+            QLatin1String(TELEPATHY_INTERFACE_CHANNEL_DISPATCHER),
+            objectPath),
+      OptionalInterfaceFactory<ChannelRequest>(this),
+      ReadyObject(this, FeatureCore),
+      mPriv(new Private(this, immutableProperties))
+{
+    mPriv->account = account;
+    mPriv->connFact = account->connectionFactory();
+    mPriv->chanFact = account->channelFactory();
+    mPriv->contactFact = account->contactFactory();
 }
 
 /**
@@ -380,29 +564,20 @@ void ChannelRequest::gotMainProperties(QDBusPendingCallWatcher *watcher)
         props = reply.value();
 
         mPriv->extractMainProps(props);
-
-        if (mPriv->account) {
-            connect(mPriv->account->becomeReady(),
-                    SIGNAL(finished(Tp::PendingOperation *)),
-                    SLOT(onAccountReady(Tp::PendingOperation *)));
-        } else {
-            warning() << "Properties.GetAll(ChannelRequest) is missing "
-                "account property, ignoring";
-            mPriv->readinessHelper->setIntrospectCompleted(FeatureCore, true);
-        }
-    }
-    else {
+    } else {
         mPriv->readinessHelper->setIntrospectCompleted(FeatureCore,
                 false, reply.error());
         warning().nospace() << "Properties::GetAll(ChannelRequest) failed with "
             << reply.error().name() << ": " << reply.error().message();
     }
+
+    watcher->deleteLater();
 }
 
 void ChannelRequest::onAccountReady(PendingOperation *op)
 {
     if (op->isError()) {
-        warning() << "Unable to make account ready";
+        warning() << "Unable to make ChannelRequest.Account ready";
         mPriv->readinessHelper->setIntrospectCompleted(FeatureCore, false,
                 op->errorName(), op->errorMessage());
         return;
