@@ -1,8 +1,8 @@
 /*
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008 Nokia Corporation
+ * Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * Copyright (C) 2008-2010 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -42,6 +42,7 @@
 namespace Tp
 {
 
+class ConnectionCapabilities;
 class PendingConnection;
 class PendingReady;
 class PendingStringList;
@@ -94,12 +95,24 @@ public:
 
     bool canRegister() const;
 
+    ConnectionCapabilities *capabilities() const;
+
+    QString vcardField() const;
+
+    QString englishName() const;
+
+    QString iconName() const;
+
 private:
     Q_DISABLE_COPY(ProtocolInfo);
 
     ProtocolInfo(const QString &cmName, const QString &name);
 
     void addParameter(const ParamSpec &spec);
+    void setRequestableChannelClasses(const RequestableChannelClassList &caps);
+    void setVCardField(const QString &vcardField);
+    void setEnglishName(const QString &englishName);
+    void setIconName(const QString &iconName);
 
     struct Private;
     friend struct Private;
@@ -131,8 +144,10 @@ public:
 
     QStringList supportedProtocols() const;
     const ProtocolInfoList &protocols() const;
+    bool hasProtocol(const QString &protocolName) const;
+    ProtocolInfo *protocol(const QString &protocolName) const;
 
-    PendingConnection *requestConnection(const QString &protocol,
+    PendingConnection *requestConnection(const QString &protocolName,
             const QVariantMap &parameters);
 
     static PendingStringList *listNames(
@@ -151,8 +166,9 @@ protected:
 
 private Q_SLOTS:
     void gotMainProperties(QDBusPendingCallWatcher *);
-    void gotProtocols(QDBusPendingCallWatcher *);
-    void gotParameters(QDBusPendingCallWatcher *);
+    void gotProtocolsLegacy(QDBusPendingCallWatcher *);
+    void gotParametersLegacy(QDBusPendingCallWatcher *);
+    void onProtocolReady(Tp::PendingOperation *);
 
 private:
     friend class PendingConnection;
