@@ -26,6 +26,7 @@
 #include "TelepathyQt4/_gen/contact-manager-internal.moc.hpp"
 
 #include <QMap>
+#include <QPointer>
 #include <QString>
 #include <QSet>
 #include <QWeakPointer>
@@ -37,6 +38,7 @@
 #include <TelepathyQt4/PendingFailure>
 #include <TelepathyQt4/PendingHandles>
 #include <TelepathyQt4/ReferencedHandles>
+#include <TelepathyQt4/Utils>
 
 #include "TelepathyQt4/debug-internal.h"
 
@@ -53,8 +55,8 @@ namespace Tp
 
 struct TELEPATHY_QT4_NO_EXPORT ContactManager::Private
 {
-    Private(ContactManager *parent, const ConnectionPtr &connection)
-        : parent(parent), connection(connection)
+    Private(ContactManager *parent, Connection *connection)
+        : parent(parent), connection(connection), requestAvatarsIdle(false)
     {
     }
 
@@ -89,7 +91,7 @@ struct TELEPATHY_QT4_NO_EXPORT ContactManager::Private
     class PendingContactManagerRemoveContactListGroup;
 
     ContactManager *parent;
-    WeakPtr<Connection> connection;
+    QPointer<Connection> connection;
     QMap<uint, QWeakPointer<Contact> > contacts;
     Contacts cachedAllKnownContacts;
 
@@ -110,7 +112,13 @@ struct TELEPATHY_QT4_NO_EXPORT ContactManager::Private
     void computeKnownContactsChanges(const Contacts &added,
         const Contacts &pendingAdded,
         const Contacts &remotePendingAdded,
-        const Contacts &removed);
+        const Contacts &removed,
+        const Channel::GroupMemberChangeDetails &details);
+
+    bool buildAvatarFileName(QString token, bool createDir,
+        QString &avatarFileName, QString &mimeTypeFileName);
+    UIntList requestAvatarsQueue;
+    bool requestAvatarsIdle;
 };
 
 ConnectionPtr ContactManager::connection() const
@@ -127,6 +135,8 @@ QString featureToInterface(Contact::Feature feature)
             return QLatin1String(TELEPATHY_INTERFACE_CONNECTION_INTERFACE_ALIASING);
         case Contact::FeatureAvatarToken:
             return QLatin1String(TELEPATHY_INTERFACE_CONNECTION_INTERFACE_AVATARS);
+        case Contact::FeatureAvatarData:
+            return QLatin1String(TELEPATHY_INTERFACE_CONNECTION_INTERFACE_AVATARS);
         case Contact::FeatureSimplePresence:
             return QLatin1String(TELEPATHY_INTERFACE_CONNECTION_INTERFACE_SIMPLE_PRESENCE);
         case Contact::FeatureCapabilities:
@@ -150,6 +160,7 @@ QSet<Contact::Feature> ContactManager::supportedFeatures() const
         QList<Contact::Feature> allFeatures = QList<Contact::Feature>()
             << Contact::FeatureAlias
             << Contact::FeatureAvatarToken
+            << Contact::FeatureAvatarData
             << Contact::FeatureSimplePresence
             << Contact::FeatureCapabilities
             << Contact::FeatureLocation
@@ -205,6 +216,8 @@ QStringList ContactManager::allKnownGroups() const
 /**
  * Attempt to add an user-defined contact list group named \a group.
  *
+ * This method requires Connection::FeatureRosterGroups to be enabled.
+ *
  * On some protocols (e.g. XMPP) empty groups are not represented on the server,
  * so disconnecting from the server and reconnecting might cause empty groups to
  * vanish.
@@ -212,6 +225,12 @@ QStringList ContactManager::allKnownGroups() const
  * The returned pending operation will finish successfully if the group already
  * exists.
  *
+ * FIXME: currently, the returned pending operation will finish as soon as the
+ * CM EnsureChannel has returned. At this point however the NewChannels
+ * mechanism hasn't yet populated our contactListGroupChannels member, which
+ * means one has to wait for groupAdded before being able to actually do
+ * something with the group (which is error-prone!). This is fd.o #29728.
+ *
  * \param group Group name.
  * \return A pending operation which will return when an attempt has been made
  *         to add an user-defined contact list group.
@@ -219,6 +238,14 @@ QStringList ContactManager::allKnownGroups() const
  */
 PendingOperation *ContactManager::addGroup(const QString &group)
 {
+    if (!connection()->isValid()) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("the Connection is invalid"), this);
+    } else if (!connection()->isReady(Connection::FeatureRosterGroups)) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("Connection::FeatureRosterGroups is not ready"), this);
+    }
+
     QVariantMap request;
     request.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".ChannelType"),
                                  QLatin1String(TELEPATHY_INTERFACE_CHANNEL_TYPE_CONTACT_LIST));
@@ -234,6 +261,13 @@ PendingOperation *ContactManager::addGroup(const QString &group)
  *
  * This method requires Connection::FeatureRosterGroups to be enabled.
  *
+ * FIXME: currently, the returned pending operation will finish as soon as the
+ * CM close() has returned. At this point however the invalidated()
+ * mechanism hasn't yet removed the channel from our contactListGroupChannels
+ * member, which means contacts can seemingly still be added to the group etc.
+ * until the change is picked up (and groupRemoved is emitted). This is fd.o
+ * #29728.
+ *
  * \param group Group name.
  * \return A pending operation which will return when an attempt has been made
  *         to remove an user-defined contact list group.
@@ -241,6 +275,14 @@ PendingOperation *ContactManager::addGroup(const QString &group)
  */
 PendingOperation *ContactManager::removeGroup(const QString &group)
 {
+    if (!connection()->isValid()) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("the Connection is invalid"), this);
+    } else if (!connection()->isReady(Connection::FeatureRosterGroups)) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("Connection::FeatureRosterGroups is not ready"), this);
+    }
+
     if (!mPriv->contactListGroupChannels.contains(group)) {
         return new PendingFailure(QLatin1String(TELEPATHY_ERROR_INVALID_ARGUMENT),
                 QLatin1String("Invalid group"), this);
@@ -287,6 +329,14 @@ Contacts ContactManager::groupContacts(const QString &group) const
 PendingOperation *ContactManager::addContactsToGroup(const QString &group,
         const QList<ContactPtr> &contacts)
 {
+    if (!connection()->isValid()) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("the Connection is invalid"), this);
+    } else if (!connection()->isReady(Connection::FeatureRosterGroups)) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("Connection::FeatureRosterGroups is not ready"), this);
+    }
+
     if (!mPriv->contactListGroupChannels.contains(group)) {
         return new PendingFailure(QLatin1String(TELEPATHY_ERROR_INVALID_ARGUMENT),
                 QLatin1String("Invalid group"), this);
@@ -310,6 +360,14 @@ PendingOperation *ContactManager::addContactsToGroup(const QString &group,
 PendingOperation *ContactManager::removeContactsFromGroup(const QString &group,
         const QList<ContactPtr> &contacts)
 {
+    if (!connection()->isValid()) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("the Connection is invalid"), this);
+    } else if (!connection()->isReady(Connection::FeatureRosterGroups)) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("Connection::FeatureRosterGroups is not ready"), this);
+    }
+
     if (!mPriv->contactListGroupChannels.contains(group)) {
         return new PendingFailure(QLatin1String(TELEPATHY_ERROR_INVALID_ARGUMENT),
                 QLatin1String("Invalid group"), this);
@@ -362,6 +420,8 @@ bool ContactManager::subscriptionRequestHasMessage() const
  * This operation is sometimes called "adding contacts to the buddy
  * list" or "requesting authorization".
  *
+ * This method requires Connection::FeatureRoster to be ready.
+ *
  * On most protocols, the contacts will need to give permission
  * before the user will be able to receive their presence: if so, they will
  * be in presence state Contact::PresenceStateAsk until they authorize
@@ -381,6 +441,14 @@ bool ContactManager::subscriptionRequestHasMessage() const
 PendingOperation *ContactManager::requestPresenceSubscription(
         const QList<ContactPtr> &contacts, const QString &message)
 {
+    if (!connection()->isValid()) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("the Connection is invalid"), this);
+    } else if (!connection()->isReady(Connection::FeatureRoster)) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("Connection::FeatureRoster is not ready"), this);
+    }
+
     if (!mPriv->subscribeChannel) {
         return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_IMPLEMENTED),
                 QLatin1String("Cannot subscribe to contacts' presence on this protocol"),
@@ -460,6 +528,8 @@ bool ContactManager::subscriptionRescindingHasMessage() const
  * Attempt to stop receiving the presence of the given contacts, or cancel
  * a request to subscribe to their presence that was previously sent.
  *
+ * This method requires Connection::FeatureRoster to be ready.
+ *
  * \param contacts Contacts whose presence is no longer required
  * \message A message from the user which is either transmitted to the
  *          contacts, or ignored, depending on the protocol
@@ -469,6 +539,14 @@ bool ContactManager::subscriptionRescindingHasMessage() const
 PendingOperation *ContactManager::removePresenceSubscription(
         const QList<ContactPtr> &contacts, const QString &message)
 {
+    if (!connection()->isValid()) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("the Connection is invalid"), this);
+    } else if (!connection()->isReady(Connection::FeatureRoster)) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("Connection::FeatureRoster is not ready"), this);
+    }
+
     if (!mPriv->subscribeChannel) {
         return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_IMPLEMENTED),
                 QLatin1String("Cannot subscribe to contacts' presence on this protocol"),
@@ -517,6 +595,8 @@ bool ContactManager::publicationAuthorizationHasMessage() const
  * If the given contacts have asked the user to publish presence to them,
  * grant permission for this publication to take place.
  *
+ * This method requires Connection::FeatureRoster to be ready.
+ *
  * \param contacts Contacts who should be allowed to receive the user's
  *                 presence
  * \message A message from the user which is either transmitted to the
@@ -527,6 +607,14 @@ bool ContactManager::publicationAuthorizationHasMessage() const
 PendingOperation *ContactManager::authorizePresencePublication(
         const QList<ContactPtr> &contacts, const QString &message)
 {
+    if (!connection()->isValid()) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("the Connection is invalid"), this);
+    } else if (!connection()->isReady(Connection::FeatureRoster)) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("Connection::FeatureRoster is not ready"), this);
+    }
+
     if (!mPriv->publishChannel) {
         return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_IMPLEMENTED),
                 QLatin1String("Cannot control publication of presence on this protocol"),
@@ -595,6 +683,8 @@ bool ContactManager::publicationRemovalHasMessage() const
  * deny this request (this should always succeed, unless a network error
  * occurs).
  *
+ * This method requires Connection::FeatureRoster to be ready.
+ *
  * If the given contacts already have permission to receive
  * the user's presence, attempt to revoke that permission (this might not
  * be supported by the protocol - canRemovePresencePublication
@@ -610,6 +700,14 @@ bool ContactManager::publicationRemovalHasMessage() const
 PendingOperation *ContactManager::removePresencePublication(
         const QList<ContactPtr> &contacts, const QString &message)
 {
+    if (!connection()->isValid()) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("the Connection is invalid"), this);
+    } else if (!connection()->isReady(Connection::FeatureRoster)) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("Connection::FeatureRoster is not ready"), this);
+    }
+
     if (!mPriv->publishChannel) {
         return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_IMPLEMENTED),
                 QLatin1String("Cannot control publication of presence on this protocol"),
@@ -634,6 +732,8 @@ bool ContactManager::canBlockContacts() const
  * messages to the user; depending on the protocol, blocking a contact may
  * have other effects.
  *
+ * This method requires Connection::FeatureRoster to be ready.
+ *
  * \param contacts Contacts who should be added to, or removed from, the list
  *                 of blocked contacts
  * \param value If true, add the contacts to the list of blocked contacts;
@@ -644,6 +744,14 @@ bool ContactManager::canBlockContacts() const
 PendingOperation *ContactManager::blockContacts(
         const QList<ContactPtr> &contacts, bool value)
 {
+    if (!connection()->isValid()) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("the Connection is invalid"), this);
+    } else if (!connection()->isReady(Connection::FeatureRoster)) {
+        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("Connection::FeatureRoster is not ready"), this);
+    }
+
     if (!mPriv->denyChannel) {
         return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_IMPLEMENTED),
                 QLatin1String("Cannot block contacts on this protocol"),
@@ -661,35 +769,45 @@ PendingOperation *ContactManager::blockContacts(
 PendingContacts *ContactManager::contactsForHandles(const UIntList &handles,
         const QSet<Contact::Feature> &features)
 {
-    debug() << "Building contacts for" << handles.size() << "handles" << "with" << features.size()
-        << "features";
-
     QMap<uint, ContactPtr> satisfyingContacts;
     QSet<uint> otherContacts;
     QSet<Contact::Feature> missingFeatures;
 
+    // FeatureAvatarData depends on FeatureAvatarToken
+    QSet<Contact::Feature> realFeatures(features);
+    if (realFeatures.contains(Contact::FeatureAvatarData) &&
+        !realFeatures.contains(Contact::FeatureAvatarToken)) {
+        realFeatures.insert(Contact::FeatureAvatarToken);
+    }
+
+    if (!connection()->isValid()) {
+        return new PendingContacts(this, handles, realFeatures, QStringList(),
+                satisfyingContacts, otherContacts, QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("the Connection is invalid"));
+    } else if (!connection()->isReady(Connection::FeatureCore)) {
+        return new PendingContacts(this, handles, realFeatures, QStringList(),
+                satisfyingContacts, otherContacts, QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("Connection::FeatureCore is not ready"));
+    }
+
     foreach (uint handle, handles) {
         ContactPtr contact = lookupContactByHandle(handle);
         if (contact) {
-            if ((features - contact->requestedFeatures()).isEmpty()) {
+            if ((realFeatures - contact->requestedFeatures()).isEmpty()) {
                 // Contact exists and has all the requested features
                 satisfyingContacts.insert(handle, contact);
             } else {
                 // Contact exists but is missing features
                 otherContacts.insert(handle);
-                missingFeatures.unite(features - contact->requestedFeatures());
+                missingFeatures.unite(realFeatures - contact->requestedFeatures());
             }
         } else {
             // Contact doesn't exist - we need to get all of the features (same as unite(features))
-            missingFeatures = features;
+            missingFeatures = realFeatures;
             otherContacts.insert(handle);
         }
     }
 
-    debug() << " " << satisfyingContacts.size() << "satisfying and"
-                   << otherContacts.size() << "other contacts";
-    debug() << " " << missingFeatures.size() << "features missing";
-
     QSet<Contact::Feature> supported = supportedFeatures();
     QSet<QString> interfaces;
     foreach (Contact::Feature feature, missingFeatures) {
@@ -702,7 +820,7 @@ PendingContacts *ContactManager::contactsForHandles(const UIntList &handles,
     }
 
     PendingContacts *contacts =
-        new PendingContacts(this, handles, features, interfaces.toList(),
+        new PendingContacts(this, handles, realFeatures, interfaces.toList(),
                 satisfyingContacts, otherContacts);
     return contacts;
 }
@@ -716,8 +834,15 @@ PendingContacts *ContactManager::contactsForHandles(const ReferencedHandles &han
 PendingContacts *ContactManager::contactsForIdentifiers(const QStringList &identifiers,
         const QSet<Contact::Feature> &features)
 {
-    debug() << "Building contacts for" << identifiers.size() << "identifiers" << "with" << features.size()
-        << "features";
+    if (!connection()->isValid()) {
+        return new PendingContacts(this, identifiers, features,
+                QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("the Connection is invalid"));
+    } else if (!connection()->isReady(Connection::FeatureCore)) {
+        return new PendingContacts(this, identifiers, features,
+                QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("Connection::FeatureCore is not ready"));
+    }
 
     PendingContacts *contacts = new PendingContacts(this, identifiers, features);
     return contacts;
@@ -726,8 +851,15 @@ PendingContacts *ContactManager::contactsForIdentifiers(const QStringList &ident
 PendingContacts *ContactManager::upgradeContacts(const QList<ContactPtr> &contacts,
         const QSet<Contact::Feature> &features)
 {
-    debug() << "Upgrading" << contacts.size() << "contacts to have at least"
-                           << features.size() << "features";
+    if (!connection()->isValid()) {
+        return new PendingContacts(this, contacts, features,
+                QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("the Connection is invalid"));
+    } else if (!connection()->isReady(Connection::FeatureCore)) {
+        return new PendingContacts(this, contacts, features,
+                QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
+                QLatin1String("Connection::FeatureCore is not ready"));
+    }
 
     return new PendingContacts(this, contacts, features);
 }
@@ -745,6 +877,76 @@ void ContactManager::onAliasesChanged(const AliasPairList &aliases)
     }
 }
 
+bool ContactManager::Private::buildAvatarFileName(QString token, bool createDir,
+    QString &avatarFileName, QString &mimeTypeFileName)
+{
+    QString cacheDir = QString(QLatin1String(qgetenv("XDG_CACHE_HOME")));
+    if (cacheDir.isEmpty()) {
+        cacheDir = QString(QLatin1String("%1/.cache")).arg(QLatin1String(qgetenv("HOME")));
+    }
+
+    QString path = QString(QLatin1String("%1/telepathy/avatars/%2/%3")).
+        arg(cacheDir).arg(connection->cmName()).arg(connection->protocolName());
+
+    if (createDir && !QDir().mkpath(path)) {
+        return false;
+    }
+
+    avatarFileName = QString(QLatin1String("%1/%2")).arg(path).arg(escapeAsIdentifier(token));
+    mimeTypeFileName = QString(QLatin1String("%1.mime")).arg(avatarFileName);
+
+    return true;
+}
+
+void ContactManager::doRequestAvatars()
+{
+    debug() << "Request" << mPriv->requestAvatarsQueue.size() << "avatar(s)";
+
+    QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(
+        mPriv->connection->avatarsInterface()->RequestAvatars(mPriv->requestAvatarsQueue),
+        this);
+    connect(watcher, SIGNAL(finished(QDBusPendingCallWatcher *)), watcher,
+        SLOT(deleteLater()));
+
+    mPriv->requestAvatarsQueue = UIntList();
+    mPriv->requestAvatarsIdle = false;
+}
+
+void ContactManager::requestContactAvatar(Contact *contact)
+{
+    QString token = contact->avatarToken();
+    QString avatarFileName;
+    QString mimeTypeFileName;
+
+    bool success = mPriv->buildAvatarFileName(token, false, avatarFileName,
+        mimeTypeFileName);
+
+    /* Check if the avatar is already in the cache */
+    if (success && QFile::exists(avatarFileName)) {
+        QFile mimeTypeFile(mimeTypeFileName);
+        mimeTypeFile.open(QIODevice::ReadOnly);
+        QString mimeType = QString(QLatin1String(mimeTypeFile.readAll()));
+        mimeTypeFile.close();
+
+        debug() << "Avatar found in cache for handle" << contact->handle()[0];
+        debug() << "Filename:" << avatarFileName;
+        debug() << "MimeType:" << mimeType;
+
+        contact->receiveAvatarData(AvatarData(avatarFileName, mimeType));
+
+        return;
+    }
+
+    /* Not found in cache, queue this contact. We do this to group contacts
+     * for the AvatarRequest call */
+    debug() << "Need to request avatar for handle" << contact->handle()[0];
+    if (!mPriv->requestAvatarsIdle) {
+        QTimer::singleShot(0, this, SLOT(doRequestAvatars()));
+        mPriv->requestAvatarsIdle = true;
+    }
+    mPriv->requestAvatarsQueue.append(contact->handle()[0]);
+}
+
 void ContactManager::onAvatarUpdated(uint handle, const QString &token)
 {
     debug() << "Got AvatarUpdate for contact with handle" << handle;
@@ -755,6 +957,41 @@ void ContactManager::onAvatarUpdated(uint handle, const QString &token)
     }
 }
 
+void ContactManager::onAvatarRetrieved(uint handle, const QString &token,
+    const QByteArray &data, const QString &mimeType)
+{
+    QString avatarFileName;
+    QString mimeTypeFileName;
+
+    debug() << "Got AvatarRetrieved for contact with handle" << handle;
+
+    bool success = mPriv->buildAvatarFileName(token, true, avatarFileName,
+        mimeTypeFileName);
+
+    if (success) {
+        QFile mimeTypeFile(mimeTypeFileName);
+        QFile avatarFile(avatarFileName);
+
+        debug() << "Write avatar in cache for handle" << handle;
+        debug() << "Filename:" << avatarFileName;
+        debug() << "MimeType:" << mimeType;
+
+        mimeTypeFile.open(QIODevice::WriteOnly);
+        mimeTypeFile.write(mimeType.toLatin1());
+        mimeTypeFile.close();
+
+        avatarFile.open(QIODevice::WriteOnly);
+        avatarFile.write(data);
+        avatarFile.close();
+    }
+
+    ContactPtr contact = lookupContactByHandle(handle);
+    if (contact) {
+        contact->setAvatarToken(token);
+        contact->receiveAvatarData(AvatarData(avatarFileName, mimeType));
+    }
+}
+
 void ContactManager::onPresencesChanged(const SimpleContactPresences &presences)
 {
     debug() << "Got PresencesChanged for" << presences.size() << "contacts";
@@ -836,7 +1073,7 @@ void ContactManager::onSubscribeChannelMembersChanged(
     mPriv->computeKnownContactsChanges(groupMembersAdded,
                                        groupLocalPendingMembersAdded,
                                        groupRemotePendingMembersAdded,
-                                       groupMembersRemoved);
+                                       groupMembersRemoved, details);
 }
 
 void ContactManager::onPublishChannelMembersChanged(
@@ -869,13 +1106,15 @@ void ContactManager::onPublishChannelMembersChanged(
 
     if (!groupLocalPendingMembersAdded.isEmpty()) {
         emit presencePublicationRequested(groupLocalPendingMembersAdded);
+        emit presencePublicationRequested(groupLocalPendingMembersAdded,
+            details);
     }
 
     // Perform the needed computation for allKnownContactsChanged
     mPriv->computeKnownContactsChanges(groupMembersAdded,
                                        groupLocalPendingMembersAdded,
                                        groupRemotePendingMembersAdded,
-                                       groupMembersRemoved);
+                                       groupMembersRemoved, details);
 }
 
 void ContactManager::onDenyChannelMembersChanged(
@@ -926,6 +1165,7 @@ void ContactManager::onContactListGroupMembersChanged(
     }
 
     emit groupMembersChanged(id, groupMembersAdded, groupMembersRemoved);
+    emit groupMembersChanged(id, groupMembersAdded, groupMembersRemoved, details);
 }
 
 void ContactManager::onContactListGroupRemoved(Tp::DBusProxy *proxy,
@@ -945,8 +1185,8 @@ void ContactManager::onContactListGroupRemoved(Tp::DBusProxy *proxy,
     emit groupRemoved(id);
 }
 
-ContactManager::ContactManager(const ConnectionPtr &connection)
-    : QObject(connection.data()),
+ContactManager::ContactManager(Connection *connection)
+    : QObject(connection),
       mPriv(new Private(this, connection))
 {
 }
@@ -998,15 +1238,13 @@ void ContactManager::setContactListChannels(
     // Refresh the cache for the current known contacts
     mPriv->cachedAllKnownContacts = allKnownContacts();
 
-    QMap<uint, ContactListChannel>::const_iterator i = contactListChannels.constBegin();
-    QMap<uint, ContactListChannel>::const_iterator end = contactListChannels.constEnd();
     uint type;
     ChannelPtr channel;
     const char *method;
-    while (i != end) {
+    for (QMap<uint, ContactListChannel>::const_iterator i = contactListChannels.constBegin();
+            i != contactListChannels.constEnd(); ++i) {
         type = i.key();
         channel = i.value().channel;
-        ++i;
         if (!channel) {
             continue;
         }
@@ -1103,6 +1341,14 @@ void ContactManager::Private::ensureTracking(Contact::Feature feature)
                     SLOT(onAvatarUpdated(uint, const QString &)));
             break;
 
+        case Contact::FeatureAvatarData:
+            QObject::connect(
+                    conn->avatarsInterface(),
+                    SIGNAL(AvatarRetrieved(uint, const QString &, const QByteArray &, const QString &)),
+                    conn->contactManager(),
+                    SLOT(onAvatarRetrieved(uint, const QString &, const QByteArray &, const QString &)));
+            break;
+
         case Contact::FeatureSimplePresence:
             QObject::connect(
                     conn->simplePresenceInterface(),
@@ -1160,9 +1406,8 @@ Contacts ContactManager::Private::allKnownContacts() const
 }
 
 void ContactManager::Private::computeKnownContactsChanges(const Tp::Contacts& added,
-                                                          const Tp::Contacts& pendingAdded,
-                                                          const Tp::Contacts& remotePendingAdded,
-                                                          const Tp::Contacts& removed)
+        const Tp::Contacts& pendingAdded, const Tp::Contacts& remotePendingAdded,
+        const Tp::Contacts& removed, const Channel::GroupMemberChangeDetails &details)
 {
     // First of all, compute the real additions/removals based upon our cache
     Tp::Contacts realAdded;
@@ -1190,6 +1435,7 @@ void ContactManager::Private::computeKnownContactsChanges(const Tp::Contacts& ad
         cachedAllKnownContacts.unite(realAdded);
         cachedAllKnownContacts.subtract(realRemoved);
         emit parent->allKnownContactsChanged(realAdded, realRemoved);
+        emit parent->allKnownContactsChanged(realAdded, realRemoved, details);
     }
 }
 
