@@ -106,13 +106,21 @@ public:
 
     ContactPtr lookupContactByHandle(uint handle);
 
+    void requestContactAvatar(Contact *contact);
+
 Q_SIGNALS:
-    void presencePublicationRequested(const Tp::Contacts &contacts);
+    void presencePublicationRequested(const Tp::Contacts &contacts); // API/ABI break: remove
+    void presencePublicationRequested(const Tp::Contacts &contacts,
+        const Tp::Channel::GroupMemberChangeDetails &details);
     void groupAdded(const QString &group);
     void groupRemoved(const QString &group);
     void groupMembersChanged(const QString &group,
             const Tp::Contacts &groupMembersAdded,
-            const Tp::Contacts &groupMembersRemoved);
+            const Tp::Contacts &groupMembersRemoved); // API/ABI break: remove
+    void groupMembersChanged(const QString &group,
+            const Tp::Contacts &groupMembersAdded,
+            const Tp::Contacts &groupMembersRemoved,
+            const Tp::Channel::GroupMemberChangeDetails &details);
     /**
      * This signal is emitted whenever some contacts got removed or added from
      * ContactManager's known contact list. It is useful for monitoring which contacts
@@ -126,11 +134,17 @@ Q_SIGNALS:
      *       over publication and/or subscription state changes if that is the case.
      */
     void allKnownContactsChanged(const Tp::Contacts &contactsAdded,
-            const Tp::Contacts &contactsRemoved);
+            const Tp::Contacts &contactsRemoved); // API/ABI break: remove
+
+    void allKnownContactsChanged(const Tp::Contacts &contactsAdded,
+            const Tp::Contacts &contactsRemoved,
+            const Tp::Channel::GroupMemberChangeDetails &details);
 
 private Q_SLOTS:
     void onAliasesChanged(const Tp::AliasPairList &);
     void onAvatarUpdated(uint, const QString &);
+    void onAvatarRetrieved(uint, const QString &, const QByteArray &, const QString &);
+    void doRequestAvatars();
     void onPresencesChanged(const Tp::SimpleContactPresences &);
     void onCapabilitiesChanged(const Tp::ContactCapabilitiesMap &);
     void onLocationUpdated(uint, const QVariantMap &);
@@ -200,7 +214,7 @@ private:
         ChannelPtr channel;
     };
 
-    ContactManager(const ConnectionPtr &parent);
+    ContactManager(Connection *parent);
     ~ContactManager();
 
     ContactPtr ensureContact(const ReferencedHandles &handle,
