@@ -19,6 +19,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
+
 #include <TelepathyQt4/DBusProxy>
 
 #include "TelepathyQt4/_gen/dbus-proxy.moc.hpp"
@@ -27,11 +29,14 @@
 
 #include <TelepathyQt4/Constants>
 
-#include <QTimer>
-
 #include <QDBusConnection>
 #include <QDBusConnectionInterface>
 #include <QDBusError>
+#include <QTimer>
+
+#ifdef HAVE_QDBUSSERVICEWATCHER
+#include <QDBusServiceWatcher>
+#endif
 
 namespace Tp
 {
@@ -242,6 +247,14 @@ void DBusProxy::emitInvalidated()
 
 // ==== StatefulDBusProxy ==============================================
 
+struct StatefulDBusProxy::Private
+{
+    Private(const QString &originalName)
+        : originalName(originalName) {}
+
+    QString originalName;
+};
+
 /**
  * \class StatefulDBusProxy
  * \ingroup FIXME: what group is it in?
@@ -257,23 +270,26 @@ void DBusProxy::emitInvalidated()
 StatefulDBusProxy::StatefulDBusProxy(const QDBusConnection &dbusConnection,
         const QString &busName, const QString &objectPath, QObject *parent)
     : DBusProxy(dbusConnection, busName, objectPath, parent),
-      mPriv(0)
+      mPriv(new Private(busName))
 {
-    QString uniqueName = busName;
+#ifdef HAVE_QDBUSSERVICEWATCHER
+    QDBusServiceWatcher *serviceWatcher = new QDBusServiceWatcher(busName,
+            dbusConnection, QDBusServiceWatcher::WatchForUnregistration, this);
+    connect(serviceWatcher,
+            SIGNAL(serviceOwnerChanged(QString, QString, QString)),
+            SLOT(onServiceOwnerChanged(QString, QString, QString)));
+#else
+    connect(dbusConnection.interface(),
+            SIGNAL(serviceOwnerChanged(QString, QString, QString)),
+            SLOT(onServiceOwnerChanged(QString, QString, QString)));
+#endif
 
-    connect(dbusConnection.interface(), SIGNAL(serviceOwnerChanged(QString, QString, QString)),
-            this, SLOT(onServiceOwnerChanged(QString, QString, QString)));
+    QString error, message;
+    QString uniqueName = uniqueNameFrom(dbusConnection, busName, error, message);
 
-    if (!busName.startsWith(QLatin1String(":"))) {
-        // For a stateful interface, it makes no sense to follow name-owner
-        // changes, so we want to bind to the unique name.
-        QDBusReply<QString> reply = dbusConnection.interface()->serviceOwner(
-                busName);
-        if (reply.isValid()) {
-            uniqueName = reply.value();
-        } else {
-            invalidate(reply.error());
-        }
+    if (uniqueName.isEmpty()) {
+        invalidate(error, message);
+        return;
     }
 
     setBusName(uniqueName);
@@ -281,13 +297,45 @@ StatefulDBusProxy::StatefulDBusProxy(const QDBusConnection &dbusConnection,
 
 StatefulDBusProxy::~StatefulDBusProxy()
 {
+    delete mPriv;
+}
+
+QString StatefulDBusProxy::uniqueNameFrom(const QDBusConnection &bus, const QString &name)
+{
+    QString error, message;
+    QString uniqueName = uniqueNameFrom(bus, name, error, message);
+    if (uniqueName.isEmpty()) {
+        warning() << "StatefulDBusProxy::uniqueNameFrom(): Failed to get unique name of" << name;
+        warning() << "  error:" << error << "message:" << message;
+    }
+
+    return uniqueName;
+}
+
+QString StatefulDBusProxy::uniqueNameFrom(const QDBusConnection &bus, const QString &name,
+        QString &error, QString &message)
+{
+    if (name.startsWith(QLatin1String(":"))) {
+        return name;
+    }
+
+    // For a stateful interface, it makes no sense to follow name-owner
+    // changes, so we want to bind to the unique name.
+    QDBusReply<QString> reply = bus.interface()->serviceOwner(name);
+    if (reply.isValid()) {
+        return reply.value();
+    } else {
+        error = reply.error().name();
+        message = reply.error().message();
+        return QString();
+    }
 }
 
 void StatefulDBusProxy::onServiceOwnerChanged(const QString &name, const QString &oldOwner, const QString &newOwner)
 {
     // We only want to invalidate this object if it is not already invalidated,
     // and its (not any other object's) name owner changed signal is emitted.
-    if (isValid() && name == busName() && newOwner == QLatin1String("")) {
+    if (isValid() && name == mPriv->originalName && newOwner.isEmpty()) {
         invalidate(QLatin1String(TELEPATHY_DBUS_ERROR_NAME_HAS_NO_OWNER),
                 QLatin1String("Name owner lost (service crashed?)"));
     }
