@@ -1,8 +1,8 @@
 /*
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008-2009 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008-2009 Nokia Corporation
+ * Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * Copyright (C) 2008-2010 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -37,11 +37,6 @@ namespace Tp
 
 struct TELEPATHY_QT4_NO_EXPORT ManagerFile::Private
 {
-    QString cmName;
-    KeyFile keyFile;
-    QHash<QString, ParamSpecList> protocolParams;
-    bool valid;
-
     Private(const QString &cnName);
 
     void init();
@@ -54,6 +49,26 @@ struct TELEPATHY_QT4_NO_EXPORT ManagerFile::Private
     ParamSpecList parameters(const QString &protocol) const;
 
     QVariant valueForKey(const QString &param, const QString &signature);
+
+    struct ProtocolInfo
+    {
+        ProtocolInfo() {}
+        ProtocolInfo(const ParamSpecList &params)
+            : params(params)
+        {
+        }
+
+        ParamSpecList params;
+        QString vcardField;
+        QString englishName;
+        QString iconName;
+        RequestableChannelClassList rccs;
+    };
+
+    QString cmName;
+    KeyFile keyFile;
+    QHash<QString, ProtocolInfo> protocolsMap;
+    bool valid;
 };
 
 ManagerFile::Private::Private(const QString &cmName)
@@ -92,7 +107,7 @@ void ManagerFile::Private::init()
         QString fileName = configDir + cmName + QLatin1String(".manager");
         if (QFile::exists(fileName)) {
             debug() << "parsing manager file" << fileName;
-            protocolParams.clear();
+            protocolsMap.clear();
             if (!parse(fileName)) {
                 warning() << "error parsing manager file" << fileName;
                 continue;
@@ -151,7 +166,7 @@ bool ManagerFile::Private::parse(const QString &fileName)
                 }
             }
 
-            protocolParams[protocol] = paramSpecList;
+            protocolsMap.insert(protocol, ProtocolInfo(paramSpecList));
 
             /* now that we have all param-* created, let's find their default values */
             foreach (param, params) {
@@ -174,11 +189,53 @@ bool ManagerFile::Private::parse(const QString &fileName)
                     if (value.type() == QVariant::Invalid) {
                         warning() << "param" << paramName
                                   << "has invalid signature";
+                        protocolsMap.clear();
                         return false;
                     }
                     spec->defaultValue = QDBusVariant(value);
                 }
             }
+
+            ProtocolInfo &info = protocolsMap[protocol];
+            info.vcardField = keyFile.value(QLatin1String("VCardField"));
+            info.englishName = keyFile.value(QLatin1String("EnglishName"));
+            if (info.englishName.isEmpty()) {
+                QStringList words = protocol.split(QLatin1Char('-'));
+                for (int i = 0; i < words.size(); ++i) {
+                    words[i][0] = words[i].at(0).toUpper();
+                }
+                info.englishName = words.join(QLatin1String(" "));
+            }
+
+            info.iconName = keyFile.value(QLatin1String("Icon"));
+            if (info.iconName.isEmpty()) {
+                info.iconName = QString(QLatin1String("im-%1")).arg(protocol);
+            }
+
+            QStringList rccGroups = keyFile.valueAsStringList(
+                    QLatin1String("RequestableChannelClasses"));
+            RequestableChannelClass rcc;
+            foreach (const QString &rccGroup, rccGroups) {
+                keyFile.setGroup(rccGroup);
+
+                foreach (const QString &key, keyFile.keys()) {
+                    int spaceIdx = key.indexOf(QLatin1String(" "));
+                    if (spaceIdx == -1) {
+                        continue;
+                    }
+
+                    QString propertyName = key.mid(0, spaceIdx);
+                    QString signature = key.mid(spaceIdx + 1);
+                    QString param = keyFile.value(key);
+                    QVariant value = valueForKey(key, signature);
+                    rcc.fixedProperties.insert(propertyName, value);
+                }
+
+                rcc.allowedProperties = keyFile.valueAsStringList(
+                        QLatin1String("allowed"));
+
+                info.rccs.append(rcc);
+            }
         }
     }
 
@@ -193,8 +250,8 @@ bool ManagerFile::Private::isValid() const
 bool ManagerFile::Private::hasParameter(const QString &protocol,
                                         const QString &paramName) const
 {
-    ParamSpecList paramSpecList = protocolParams[protocol];
-    foreach (ParamSpec paramSpec, paramSpecList) {
+    ParamSpecList paramSpecList = protocolsMap[protocol].params;
+    foreach (const ParamSpec &paramSpec, paramSpecList) {
         if (paramSpec.name == paramName) {
             return true;
         }
@@ -205,7 +262,7 @@ bool ManagerFile::Private::hasParameter(const QString &protocol,
 ParamSpec *ManagerFile::Private::getParameter(const QString &protocol,
                                               const QString &paramName)
 {
-    ParamSpecList &paramSpecList = protocolParams[protocol];
+    ParamSpecList &paramSpecList = protocolsMap[protocol].params;
     for (int i = 0; i < paramSpecList.size(); ++i) {
         ParamSpec &paramSpec = paramSpecList[i];
         if (paramSpec.name == paramName) {
@@ -217,12 +274,12 @@ ParamSpec *ManagerFile::Private::getParameter(const QString &protocol,
 
 QStringList ManagerFile::Private::protocols() const
 {
-    return protocolParams.keys();
+    return protocolsMap.keys();
 }
 
 ParamSpecList ManagerFile::Private::parameters(const QString &protocol) const
 {
-    return protocolParams.value(protocol);
+    return protocolsMap.value(protocol).params;
 }
 
 QVariant ManagerFile::Private::valueForKey(const QString &param,
@@ -333,7 +390,7 @@ QStringList ManagerFile::protocols() const
 }
 
 /**
- * Return a list of all protocols defined in the manager file.
+ * Return a list of parameters for the given \a protocol.
  *
  * \param protocol Name of the protocol to look for.
  * \return List of ParamSpec of a specific protocol defined in the file, or an
@@ -344,27 +401,88 @@ ParamSpecList ManagerFile::parameters(const QString &protocol) const
     return mPriv->parameters(protocol);
 }
 
+/**
+ * Return the name of the most common vCard field used for the given \a protocol's
+ * contact identifiers, normalized to lower case.
+ *
+ * \param protocol Name of the protocol to look for.
+ * \return The most common vCard field used for the given protocol's contact
+ *         identifiers, or an empty string if there is no such field or the
+ *         protocol is not defined.
+ */
+QString ManagerFile::vcardField(const QString &protocol) const
+{
+    return mPriv->protocolsMap[protocol].vcardField;
+}
+
+/**
+ * Return the English-language name of the given \a protocol, such as "AIM" or "Yahoo!".
+ *
+ * The name can be used as a fallback if an application doesn't have a localized name for the
+ * protocol.
+ *
+ * If the manager file doesn't specify the english name, it is inferred from the protocol name, such
+ * that for example "google-talk" becomes "Google Talk", but "local-xmpp" becomes "Local Xmpp".
+ *
+ * \param protocol Name of the protocol to look for.
+ * \return An English-language name for the given \a protocol.
+ */
+QString ManagerFile::englishName(const QString &protocol) const
+{
+    return mPriv->protocolsMap[protocol].englishName;
+}
+
+/**
+ * Return the name of an icon for the given \a protocol in the system's icon
+ * theme, such as "im-msn".
+ *
+ * If the manager file doesn't specify the icon name, "im-<protocolname>" is assumed.
+ *
+ * \param protocol Name of the protocol to look for.
+ * \return The likely name of an icon for the given \a protocol.
+ */
+QString ManagerFile::iconName(const QString &protocol) const
+{
+    return mPriv->protocolsMap[protocol].iconName;
+}
+
+/**
+ * Return a list of channel classes which might be requestable from a connection
+ * to the given \a protocol.
+ *
+ * \param protocol Name of the protocol to look for.
+ * \return A list of channel classes which might be requestable from a
+ *         connection to the given \a protocol or a default constructed
+ *         RequestableChannelClassList instance if the protocol is not defined.
+ */
+RequestableChannelClassList ManagerFile::requestableChannelClasses(
+        const QString &protocol) const
+{
+    return mPriv->protocolsMap[protocol].rccs;
+}
+
 QVariant::Type ManagerFile::variantTypeFromDBusSignature(const QString &signature)
 {
     QVariant::Type type;
-    if (signature == QLatin1String("b"))
+    if (signature == QLatin1String("b")) {
         type = QVariant::Bool;
-    else if (signature == QLatin1String("n") || signature == QLatin1String("i"))
+    } else if (signature == QLatin1String("n") || signature == QLatin1String("i")) {
         type = QVariant::Int;
-    else if (signature == QLatin1String("q") || signature == QLatin1String("u"))
+    } else if (signature == QLatin1String("q") || signature == QLatin1String("u")) {
         type = QVariant::UInt;
-    else if (signature == QLatin1String("x"))
+    } else if (signature == QLatin1String("x")) {
         type = QVariant::LongLong;
-    else if (signature == QLatin1String("t"))
+    } else if (signature == QLatin1String("t")) {
         type = QVariant::ULongLong;
-    else if (signature == QLatin1String("d"))
+    } else if (signature == QLatin1String("d")) {
         type = QVariant::Double;
-    else if (signature == QLatin1String("as"))
+    } else if (signature == QLatin1String("as")) {
         type = QVariant::StringList;
-    else if (signature == QLatin1String("s") || signature == QLatin1String("o"))
+    } else if (signature == QLatin1String("s") || signature == QLatin1String("o")) {
         type = QVariant::String;
-    else
+    } else {
         type = QVariant::Invalid;
+    }
 
     return type;
 }
