@@ -26,6 +26,7 @@
 #include "TelepathyQt4/debug-internal.h"
 
 #include <TelepathyQt4/AccountManager>
+#include <TelepathyQt4/PendingReady>
 
 #include <QDBusObjectPath>
 #include <QDBusPendingCallWatcher>
@@ -41,7 +42,7 @@ struct TELEPATHY_QT4_NO_EXPORT PendingAccount::Private
     {
     }
 
-    WeakPtr<AccountManager> manager;
+    AccountManagerPtr manager;
     AccountPtr account;
     QDBusObjectPath objectPath;
 };
@@ -77,7 +78,7 @@ PendingAccount::PendingAccount(const AccountManagerPtr &manager,
         const QString &connectionManager, const QString &protocol,
         const QString &displayName, const QVariantMap &parameters,
         const QVariantMap &properties)
-    : PendingOperation(manager.data()),
+    : PendingOperation(0),
       mPriv(new Private(manager))
 {
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(
@@ -103,7 +104,7 @@ PendingAccount::~PendingAccount()
  */
 AccountManagerPtr PendingAccount::manager() const
 {
-    return AccountManagerPtr(mPriv->manager);
+    return mPriv->manager;
 }
 
 /**
@@ -123,12 +124,6 @@ AccountPtr PendingAccount::account() const
         return AccountPtr();
     }
 
-    if (!mPriv->account) {
-        AccountManagerPtr manager(mPriv->manager);
-        mPriv->account = Account::create(manager->dbusConnection(),
-                manager->busName(), mPriv->objectPath.path());
-    }
-
     return mPriv->account;
 }
 
@@ -144,9 +139,9 @@ AccountPtr PendingAccount::account() const
 QString PendingAccount::objectPath() const
 {
     if (!isFinished()) {
-        warning() << "PendingAccount::account called before finished";
+        warning() << "PendingAccount::objectPath() called before finished";
     } else if (!isValid()) {
-        warning() << "PendingAccount::account called when not valid";
+        warning() << "PendingAccount::objectPath() called when not valid";
     }
 
     return mPriv->objectPath.path();
@@ -160,7 +155,13 @@ void PendingAccount::onCallFinished(QDBusPendingCallWatcher *watcher)
         mPriv->objectPath = reply.value();
         debug() << "Got reply to AccountManager.CreateAccount - object path:" <<
             mPriv->objectPath.path();
-        setFinished();
+        PendingReady *proxyOp = manager()->accountFactory()->proxy(manager()->busName(),
+                mPriv->objectPath.path(), manager()->connectionFactory(),
+                manager()->channelFactory(), manager()->contactFactory());
+        mPriv->account = AccountPtr::dynamicCast(proxyOp->proxy());
+        connect(proxyOp,
+                SIGNAL(finished(Tp::PendingOperation*)),
+                SLOT(onAccountBuilt(Tp::PendingOperation*)));
     } else {
         debug().nospace() <<
             "CreateAccount failed: " <<
@@ -171,4 +172,40 @@ void PendingAccount::onCallFinished(QDBusPendingCallWatcher *watcher)
     watcher->deleteLater();
 }
 
+void PendingAccount::onAccountBuilt(Tp::PendingOperation *op)
+{
+    Q_ASSERT(op->isFinished());
+
+    if (op->isError()) {
+        warning() << "Making account ready using the factory failed:" <<
+            op->errorName() << op->errorMessage();
+        setFinishedWithError(op->errorName(), op->errorMessage());
+    } else {
+        // AM is stateless, so the only way for it to become invalid is in the introspection phase,
+        // and a PendingAccount should never be created if AM introspection hasn't succeeded
+        Q_ASSERT(!mPriv->manager.isNull() && mPriv->manager->isValid());
+
+        if (mPriv->manager->allAccounts().contains(mPriv->account)) {
+            setFinished();
+            debug() << "New account" << objectPath() << "built";
+        } else {
+            // Have to wait for the AM to pick up the change and signal it so the world can be
+            // assumed to be ~round when we finish
+            connect(mPriv->manager.data(),
+                    SIGNAL(newAccount(Tp::AccountPtr)),
+                    SLOT(onNewAccount(Tp::AccountPtr)));
+        }
+    }
+}
+
+void PendingAccount::onNewAccount(const AccountPtr &account)
+{
+    if (account != mPriv->account) {
+        return;
+    }
+
+    debug() << "Account" << objectPath() << "added to AM, finishing PendingAccount";
+    setFinished();
+}
+
 } // Tp
