@@ -40,7 +40,7 @@ struct TELEPATHY_QT4_NO_EXPORT PendingChannel::Private
     {
     }
 
-    WeakPtr<Connection> connection;
+    ConnectionPtr connection;
     bool yours;
     QString channelType;
     uint handleType;
@@ -69,7 +69,7 @@ struct TELEPATHY_QT4_NO_EXPORT PendingChannel::Private
  */
 PendingChannel::PendingChannel(const ConnectionPtr &connection, const QString &errorName,
         const QString &errorMessage)
-    : PendingOperation(connection.data()),
+    : PendingOperation(0),
       mPriv(new Private(connection))
 {
     mPriv->yours = false;
@@ -88,7 +88,7 @@ PendingChannel::PendingChannel(const ConnectionPtr &connection, const QString &e
  */
 PendingChannel::PendingChannel(const ConnectionPtr &connection,
         const QVariantMap &request, bool create)
-    : PendingOperation(connection.data()),
+    : PendingOperation(0),
       mPriv(new Private(connection))
 {
     mPriv->yours = create;
@@ -127,7 +127,7 @@ PendingChannel::~PendingChannel()
  */
 ConnectionPtr PendingChannel::connection() const
 {
-    return ConnectionPtr(mPriv->connection);
+    return mPriv->connection;
 }
 
 /**
@@ -215,7 +215,32 @@ uint PendingChannel::targetHandle() const
  */
 QVariantMap PendingChannel::immutableProperties() const
 {
-    return mPriv->immutableProperties;
+    QVariantMap props = mPriv->immutableProperties;
+
+    // This is a reasonable guess - if it's Yours it's guaranteedly Requested by us, and if it's not
+    // it could be either Requested by somebody else but also an incoming channel just as well.
+    if (!props.contains(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".Requested"))) {
+        debug() << "CM didn't provide Requested in channel immutable props, guessing"
+            << mPriv->yours;
+        props[QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".Requested")] =
+            mPriv->yours;
+    }
+
+    // Also, the spec says that if the channel was Requested by the local user, InitiatorHandle must
+    // be the Connection's self handle
+    if (!props.contains(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".InitiatorHandle"))) {
+        if (qdbus_cast<bool>(props.value(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".Requested")))) {
+            ConnectionPtr conn(mPriv->connection);
+            if (conn && conn->isReady()) {
+                debug() << "CM didn't provide InitiatorHandle in channel immutable props, but we "
+                    "know it's the conn's self handle (and have it)";
+                props[QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".InitiatorHandle")] =
+                    conn->selfHandle();
+            }
+        }
+    }
+
+    return props;
 }
 
 /**
@@ -242,7 +267,7 @@ ChannelPtr PendingChannel::channel() const
 
     SharedPtr<Connection> conn(mPriv->connection);
     mPriv->channel = ChannelFactory::create(conn,
-            mPriv->objectPath.path(), mPriv->immutableProperties);
+            mPriv->objectPath.path(), immutableProperties());
     return mPriv->channel;
 }
 
