@@ -101,6 +101,7 @@ struct TELEPATHY_QT4_NO_EXPORT ReadinessHelper::Private
             const QString &errorName = QString(),
             const QString &errorMessage = QString());
     void iterateIntrospection();
+    Features depsFor(const Feature &feature); // Recursive dependencies for a feature
 
     void abortOperations(const QString &errorName, const QString &errorMessage);
 
@@ -137,15 +138,13 @@ ReadinessHelper::Private::Private(
       pendingStatusChange(false),
       pendingStatus(-1)
 {
-    Introspectables::const_iterator i = introspectables.constBegin();
-    Introspectables::const_iterator end = introspectables.constEnd();
-    while (i != end) {
+    for (Introspectables::const_iterator i = introspectables.constBegin();
+            i != introspectables.constEnd(); ++i) {
         Feature feature = i.key();
         Introspectable introspectable = i.value();
         Q_ASSERT(introspectable.mPriv->introspectFunc != 0);
         supportedStatuses += introspectable.mPriv->makesSenseForStatuses;
         supportedFeatures += feature;
-        ++i;
     }
 
     if (proxy) {
@@ -165,6 +164,10 @@ ReadinessHelper::Private::~Private()
 
 void ReadinessHelper::Private::setCurrentStatus(uint newStatus)
 {
+    if (currentStatus == newStatus) {
+        return;
+    }
+
     if (inFlightFeatures.isEmpty()) {
         currentStatus = newStatus;
         satisfiedFeatures.clear();
@@ -241,19 +244,16 @@ void ReadinessHelper::Private::iterateIntrospection()
 
     // take care to flag anything with dependencies in missing, and the
     // stuff depending on them, as missing
-    Introspectables::const_iterator i = introspectables.constBegin();
-    Introspectables::const_iterator end = introspectables.constEnd();
-    while (i != end) {
+    for (Introspectables::const_iterator i = introspectables.constBegin();
+            i != introspectables.constEnd(); ++i) {
         Feature feature = i.key();
         Introspectable introspectable = i.value();
-        Features dependsOnFeatures = introspectable.mPriv->dependsOnFeatures;
-        if (!dependsOnFeatures.intersect(missingFeatures).isEmpty()) {
+        if (!depsFor(feature).intersect(missingFeatures).isEmpty()) {
             missingFeatures.insert(feature);
             missingFeaturesErrors.insert(feature,
                     QPair<QString, QString>(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
-                        QLatin1String("Feature depend on other features that are not available")));
+                        QLatin1String("Feature depends on other features that are not available")));
         }
-        ++i;
     }
 
     // check if any pending operations for becomeReady should finish now
@@ -277,6 +277,13 @@ void ReadinessHelper::Private::iterateIntrospection()
         return;
     }
 
+    // Update pendingFeatures to contain the (recursive) dependencies of what's in it currently
+    Features deps;
+    foreach (Feature feature, pendingFeatures) {
+        deps.unite(depsFor(feature));
+    }
+    pendingFeatures.unite(deps);
+
     // update pendingFeatures with the difference of requested and
     // satisfied + missing
     pendingFeatures -= (satisfiedFeatures + missingFeatures);
@@ -329,6 +336,18 @@ void ReadinessHelper::Private::iterateIntrospection()
     }
 }
 
+Features ReadinessHelper::Private::depsFor(const Feature &feature)
+{
+    Features deps;
+
+    foreach (Feature dep, introspectables[feature].mPriv->dependsOnFeatures) {
+        deps += dep;
+        deps += depsFor(dep);
+    }
+
+    return deps;
+}
+
 void ReadinessHelper::Private::abortOperations(const QString &errorName,
         const QString &errorMessage)
 {
@@ -364,9 +383,8 @@ void ReadinessHelper::addIntrospectables(const Introspectables &introspectables)
 {
     // QMap::unite will create multiple items if the key is already in the map
     // so let's make sure we don't duplicate keys
-    Introspectables::const_iterator i = introspectables.constBegin();
-    Introspectables::const_iterator end = introspectables.constEnd();
-    while (i != end) {
+    for (Introspectables::const_iterator i = introspectables.constBegin();
+            i != introspectables.constEnd(); ++i) {
         Feature feature = i.key();
         if (mPriv->introspectables.contains(feature)) {
             warning() << "ReadinessHelper::addIntrospectables: trying to add an "
@@ -378,8 +396,6 @@ void ReadinessHelper::addIntrospectables(const Introspectables &introspectables)
             mPriv->supportedStatuses += introspectable.mPriv->makesSenseForStatuses;
             mPriv->supportedFeatures += feature;
         }
-
-        ++i;
     }
 
     debug() << "ReadinessHelper: new supportedStatuses =" << mPriv->supportedStatuses;
@@ -396,6 +412,23 @@ void ReadinessHelper::setCurrentStatus(uint currentStatus)
     mPriv->setCurrentStatus(currentStatus);
 }
 
+/**
+ * Force the current internal status to \a currentStatus.
+ *
+ * Note that this method will not start a new introspection or restart the
+ * current one in case one is running.
+ *
+ * This is useful for example when the status is unknown initially but it will
+ * become known in the first introspection run and there is no need to re-run
+ * the introspection.
+ *
+ * \param currentStatus The status to set.
+ */
+void ReadinessHelper::forceCurrentStatus(uint currentStatus)
+{
+    mPriv->currentStatus = currentStatus;
+}
+
 QStringList ReadinessHelper::interfaces() const
 {
     return mPriv->interfaces;
@@ -448,13 +481,11 @@ bool ReadinessHelper::isReady(const Feature &feature,
 
     if (feature.isCritical()) {
         if (!mPriv->satisfiedFeatures.contains(feature)) {
-            debug() << "ReadinessHelper::isReady: critical feature" << feature << "not ready";
             ret = false;
         }
     } else {
         if (!mPriv->satisfiedFeatures.contains(feature) &&
             !mPriv->missingFeatures.contains(feature)) {
-            debug() << "ReadinessHelper::isReady: feature" << feature << "not ready";
             ret = false;
         }
     }
@@ -500,7 +531,7 @@ PendingReady *ReadinessHelper::becomeReady(const Features &requestedFeatures)
 
     Features supportedFeatures = mPriv->supportedFeatures;
     if (supportedFeatures.intersect(requestedFeatures) != requestedFeatures) {
-        debug() << "ReadinessHelper::becomeReady called with invalid features: requestedFeatures =" <<
+        warning() << "ReadinessHelper::becomeReady called with invalid features: requestedFeatures =" <<
             requestedFeatures << "- supportedFeatures =" << mPriv->supportedFeatures;
         PendingReady *operation =
             new PendingReady(requestedFeatures, mPriv->object, this);
