@@ -40,17 +40,25 @@ TELEPATHY_QT4_EXPORT void registerTypes();
 
 class AbstractClient;
 class Account;
+class AccountPropertyFilter;
+class AccountFactory;
 class AccountSet;
 class AccountManager;
 class Channel;
 class ChannelDispatchOperation;
+class ChannelFactory;
 class ChannelRequest;
 class ClientObject;
 class ClientRegistrar;
 class Connection;
+class ConnectionFactory;
 class ConnectionManager;
 class Contact;
+class ContactFactory;
 class FileTransferChannel;
+template <typename T> class Filter;
+template <typename T> class GenericCapabilityFilter;
+template <typename T> class GenericPropertyFilter;
 class IncomingFileTransferChannel;
 class OutgoingFileTransferChannel;
 class RoomListChannel;
@@ -59,18 +67,35 @@ class MediaStream;
 class StreamedMediaChannel;
 class TextChannel;
 
+typedef Filter<Account> AccountFilter;
+typedef GenericCapabilityFilter<Account> AccountCapabilityFilter;
+
 typedef SharedPtr<AbstractClient> AbstractClientPtr;
 typedef SharedPtr<Account> AccountPtr;
+typedef SharedPtr<AccountFactory> AccountFactoryPtr;
+typedef SharedPtr<const AccountFactory> AccountFactoryConstPtr;
+typedef SharedPtr<AccountCapabilityFilter> AccountCapabilityFilterPtr;
+typedef SharedPtr<const AccountCapabilityFilter> AccountCapabilityFilterConstPtr;
+typedef SharedPtr<AccountPropertyFilter> AccountPropertyFilterPtr;
+typedef SharedPtr<const AccountPropertyFilter> AccountPropertyFilterConstPtr;
+typedef SharedPtr<AccountFilter> AccountFilterPtr;
+typedef SharedPtr<const AccountFilter> AccountFilterConstPtr;
 typedef SharedPtr<AccountSet> AccountSetPtr;
 typedef SharedPtr<AccountManager> AccountManagerPtr;
 typedef SharedPtr<Channel> ChannelPtr;
 typedef SharedPtr<ChannelDispatchOperation> ChannelDispatchOperationPtr;
+typedef SharedPtr<ChannelFactory> ChannelFactoryPtr;
+typedef SharedPtr<const ChannelFactory> ChannelFactoryConstPtr;
 typedef SharedPtr<ChannelRequest> ChannelRequestPtr;
 typedef SharedPtr<ClientObject> ClientObjectPtr;
 typedef SharedPtr<ClientRegistrar> ClientRegistrarPtr;
 typedef SharedPtr<Connection> ConnectionPtr;
+typedef SharedPtr<ConnectionFactory> ConnectionFactoryPtr;
+typedef SharedPtr<const ConnectionFactory> ConnectionFactoryConstPtr;
 typedef SharedPtr<ConnectionManager> ConnectionManagerPtr;
 typedef QSharedPointer<Contact> ContactPtr;
+typedef SharedPtr<ContactFactory> ContactFactoryPtr;
+typedef SharedPtr<const ContactFactory> ContactFactoryConstPtr;
 typedef SharedPtr<FileTransferChannel> FileTransferChannelPtr;
 typedef SharedPtr<IncomingFileTransferChannel> IncomingFileTransferChannelPtr;
 typedef SharedPtr<OutgoingFileTransferChannel> OutgoingFileTransferChannelPtr;
@@ -96,6 +121,16 @@ public:
         : SharedPtr<MethodInvocationContext<T1, T2, T3, T4, T5, T6, T7, T8> >(o) { }
 };
 
+class AvatarData
+{
+public:
+    inline AvatarData(const QString &fileName, const QString &mimeType):
+        fileName(fileName), mimeType(mimeType) {}
+    inline AvatarData() {}
+    QString fileName;
+    QString mimeType;
+};
+
 } // Tp
 
 #endif
