@@ -154,7 +154,7 @@ void CallWindow::onConnectionInvalidated(DBusProxy *proxy,
 void CallWindow::onNewChannels(const Tp::ChannelDetailsList &channels)
 {
     qDebug() << "CallWindow::onNewChannels";
-    foreach (const Tp::ChannelDetails &details, channels) {
+    Q_FOREACH (const Tp::ChannelDetails &details, channels) {
         QString channelType = details.properties.value(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".ChannelType")).toString();
         bool requested = details.properties.value(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".Requested")).toBool();
         qDebug() << " channelType:" << channelType;
