@@ -184,7 +184,7 @@ void TestChanBasics::initTestCase()
             EXAMPLE_TYPE_ECHO_2_CONNECTION,
             "account", "me@example.com",
             "protocol", "contacts",
-            0));
+            NULL));
     QVERIFY(mConnService != 0);
     QVERIFY(tp_base_connection_register(TP_BASE_CONNECTION(mConnService),
                 "foo", &name, &connPath, &error));
@@ -277,7 +277,7 @@ void TestChanBasics::testCreateChannel()
         QCOMPARE(mChan->groupSelfContact(), mConn->selfContact());
 
         QStringList ids;
-        foreach (const ContactPtr &contact, mChan->groupContacts()) {
+        Q_FOREACH (const ContactPtr &contact, mChan->groupContacts()) {
             ids << contact->id();
         }
         ids.sort();
@@ -318,7 +318,7 @@ void TestChanBasics::testEnsureChannel()
         QCOMPARE(mChan->groupSelfContact(), mConn->selfContact());
 
         QStringList ids;
-        foreach (const ContactPtr &contact, mChan->groupContacts()) {
+        Q_FOREACH (const ContactPtr &contact, mChan->groupContacts()) {
             ids << contact->id();
         }
         ids.sort();
