@@ -90,7 +90,7 @@ void TestConferenceChan::initTestCase()
             EXAMPLE_TYPE_ECHO_CONNECTION,
             "account", "me@example.com",
             "protocol", "example",
-            0));
+            NULL));
     QVERIFY(mConnService != 0);
     mBaseConnService = TP_BASE_CONNECTION(mConnService);
     QVERIFY(mBaseConnService != 0);
@@ -192,13 +192,13 @@ void TestConferenceChan::testConference()
     expectedObjectPaths << mTextChan1Path << mTextChan2Path;
 
     QStringList objectPaths;
-    foreach (const ChannelPtr &channel, mChan->conferenceInitialChannels()) {
+    Q_FOREACH (const ChannelPtr &channel, mChan->conferenceInitialChannels()) {
         objectPaths << channel->objectPath();
     }
     QCOMPARE(expectedObjectPaths, objectPaths);
 
     objectPaths.clear();
-    foreach (const ChannelPtr &channel, mChan->conferenceChannels()) {
+    Q_FOREACH (const ChannelPtr &channel, mChan->conferenceChannels()) {
         objectPaths << channel->objectPath();
     }
     QCOMPARE(expectedObjectPaths, objectPaths);
@@ -226,10 +226,13 @@ void TestConferenceChan::testConference()
 
     expectedObjectPaths << mTextChan3Path;
     objectPaths.clear();
-    foreach (const ChannelPtr &channel, mChan->conferenceChannels()) {
+    Q_FOREACH (const ChannelPtr &channel, mChan->conferenceChannels()) {
         objectPaths << channel->objectPath();
     }
     QCOMPARE(expectedObjectPaths, objectPaths);
+
+    mChan.reset();
+    mChannelMerged.reset();
 }
 
 void TestConferenceChan::cleanup()
