@@ -16,8 +16,10 @@
 #include <TelepathyQt4/Debug>
 
 #include <telepathy-glib/debug.h>
+#include <telepathy-glib/handle.h>
 
 #include <tests/lib/glib/contactlist/conn.h>
+#include <tests/lib/glib/textchan-group.h>
 #include <tests/lib/test.h>
 
 using namespace Tp;
@@ -28,7 +30,7 @@ class TestChanGroup : public Test
 
 public:
     TestChanGroup(QObject *parent = 0)
-        : Test(parent), mConnService(0)
+        : Test(parent), mConnService(0), mChanService(0)
     { }
 
 protected Q_SLOTS:
@@ -48,6 +50,8 @@ private Q_SLOTS:
 
     void testCreateContacts();
     void testCreateChannel();
+    void testMCDGroup();
+    void testPropertylessGroup();
 
     void cleanup();
     void cleanupTestCase();
@@ -55,8 +59,11 @@ private Q_SLOTS:
 private:
     void debugContacts();
 
+    void commonTest(gboolean properties);
+
     QString mConnName, mConnPath;
     ExampleContactListConnection *mConnService;
+    TpTestsTextChannelGroup *mChanService;
     ConnectionPtr mConn;
     ChannelPtr mChan;
     QString mChanObjectPath;
@@ -66,6 +73,7 @@ private:
     Contacts mChangedRP;
     Contacts mChangedRemoved;
     Channel::GroupMemberChangeDetails mDetails;
+    UIntList mInitialMembers;
 };
 
 void TestChanGroup::expectConnInvalidated()
@@ -148,17 +156,17 @@ void TestChanGroup::onGroupMembersChanged(
 void TestChanGroup::debugContacts()
 {
     qDebug() << "contacts on group:";
-    foreach (const ContactPtr &contact, mChan->groupContacts()) {
+    Q_FOREACH (const ContactPtr &contact, mChan->groupContacts()) {
         qDebug() << " " << contact->id();
     }
 
     qDebug() << "local pending contacts on group:";
-    foreach (const ContactPtr &contact, mChan->groupLocalPendingContacts()) {
+    Q_FOREACH (const ContactPtr &contact, mChan->groupLocalPendingContacts()) {
         qDebug() << " " << contact->id();
     }
 
     qDebug() << "remote pending contacts on group:";
-    foreach (const ContactPtr &contact, mChan->groupRemotePendingContacts()) {
+    Q_FOREACH (const ContactPtr &contact, mChan->groupRemotePendingContacts()) {
         qDebug() << " " << contact->id();
     }
 }
@@ -180,7 +188,8 @@ void TestChanGroup::initTestCase()
             EXAMPLE_TYPE_CONTACT_LIST_CONNECTION,
             "account", "me@example.com",
             "protocol", "example-contact-list",
-            0));
+            "simulation-delay", 1,
+            NULL));
     QVERIFY(mConnService != 0);
     QVERIFY(tp_base_connection_register(TP_BASE_CONNECTION(mConnService),
                 "foo", &name, &connPath, &error));
@@ -212,6 +221,12 @@ void TestChanGroup::initTestCase()
 void TestChanGroup::init()
 {
     initImpl();
+
+    mChangedCurrent.clear();
+    mChangedLP.clear();
+    mChangedRP.clear();
+    mChangedRemoved.clear();
+    mDetails = Channel::GroupMemberChangeDetails();
 }
 
 void TestChanGroup::testCreateContacts()
@@ -256,6 +271,9 @@ void TestChanGroup::testCreateChannel()
     QCOMPARE(mChan->isRequested(), false);
     QVERIFY(mChan->groupContacts().contains(mContacts.first()));
 
+    Q_FOREACH (ContactPtr contact, mChan->groupContacts())
+        mInitialMembers.push_back(contact->handle()[0]);
+
     QCOMPARE(mChan->groupCanAddContacts(), true);
     QCOMPARE(mChan->groupCanRemoveContacts(), true);
 
@@ -292,8 +310,99 @@ void TestChanGroup::testCreateChannel()
     QCOMPARE(mChan->groupContacts().count(), 3);
 }
 
+void TestChanGroup::testMCDGroup()
+{
+    commonTest(true);
+}
+
+void TestChanGroup::testPropertylessGroup()
+{
+    commonTest(false);
+}
+
+void TestChanGroup::commonTest(gboolean properties)
+{
+    mChanObjectPath = QString(QLatin1String("%1/ChannelForTpQt4MCDTest%2"))
+        .arg(mConn->objectPath())
+        .arg(QLatin1String(properties ? "props" : ""));
+    QByteArray chanPathLatin1(mChanObjectPath.toLatin1());
+
+    mChanService = TP_TESTS_TEXT_CHANNEL_GROUP(g_object_new(
+                TP_TESTS_TYPE_TEXT_CHANNEL_GROUP,
+                "connection", mConnService,
+                "object-path", chanPathLatin1.data(),
+                "detailed", TRUE,
+                "properties", properties,
+                NULL));
+    QVERIFY(mChanService != 0);
+
+    TpIntSet *members = tp_intset_sized_new(mInitialMembers.length());
+    Q_FOREACH (uint handle, mInitialMembers)
+        tp_intset_add(members, handle);
+
+    QVERIFY(tp_group_mixin_change_members(G_OBJECT(mChanService), "be there or be []",
+                members, NULL, NULL, NULL, 0, TP_CHANNEL_GROUP_CHANGE_REASON_NONE));
+
+    tp_intset_destroy(members);
+
+    mChan = Channel::create(mConn, mChanObjectPath, QVariantMap());
+    QVERIFY(mChan);
+
+    QVERIFY(connect(mChan->becomeReady(),
+                    SIGNAL(finished(Tp::PendingOperation*)),
+                    SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
+    QCOMPARE(mLoop->exec(), 0);
+    QCOMPARE(mChan->isReady(), true);
+
+    QCOMPARE(mChan->isRequested(), true);
+    QVERIFY(mChan->groupContacts().contains(mContacts.first()));
+
+    Q_FOREACH (ContactPtr contact, mChan->groupContacts())
+        mInitialMembers.push_back(contact->handle()[0]);
+
+    QCOMPARE(mChan->groupCanAddContacts(), false);
+    QCOMPARE(mChan->groupCanRemoveContacts(), false);
+
+    debugContacts();
+
+    QCOMPARE(mChan->groupContacts().count(), 4);
+
+    QVERIFY(connect(mChan.data(),
+                    SIGNAL(groupMembersChanged(
+                            const Tp::Contacts &,
+                            const Tp::Contacts &,
+                            const Tp::Contacts &,
+                            const Tp::Contacts &,
+                            const Tp::Channel::GroupMemberChangeDetails &)),
+                    SLOT(onGroupMembersChanged(
+                            const Tp::Contacts &,
+                            const Tp::Contacts &,
+                            const Tp::Contacts &,
+                            const Tp::Contacts &,
+                            const Tp::Channel::GroupMemberChangeDetails &))));
+
+    TpIntSet *remove = tp_intset_new_containing(mContacts[0]->handle()[0]);
+
+    QVERIFY(tp_group_mixin_change_members(G_OBJECT(mChanService), "be a []",
+                NULL, remove, NULL, NULL, 0, TP_CHANNEL_GROUP_CHANGE_REASON_NONE));
+
+    tp_intset_destroy(remove);
+
+    while (mChangedRemoved.isEmpty()) {
+        QCOMPARE(mLoop->exec(), 0);
+    }
+    QVERIFY(mChangedRemoved.contains(mContacts[0]));
+
+    QCOMPARE(mChan->groupContacts().count(), 3);
+}
+
 void TestChanGroup::cleanup()
 {
+    if (mChanService) {
+        g_object_unref(mChanService);
+        mChanService = 0;
+    }
+
     cleanupImpl();
 }
 
