@@ -61,7 +61,7 @@ class ChannelRequestAdaptor : public QDBusAbstractAdaptor
     Q_PROPERTY(QDBusObjectPath Account READ Account)
     Q_PROPERTY(qulonglong UserActionTime READ UserActionTime)
     Q_PROPERTY(QString PreferredHandler READ PreferredHandler)
-    Q_PROPERTY(QualifiedPropertyValueMapList Requests READ Requests)
+    Q_PROPERTY(Tp::QualifiedPropertyValueMapList Requests READ Requests)
     Q_PROPERTY(QStringList Interfaces READ Interfaces)
 
 public:
@@ -129,6 +129,73 @@ private:
     QStringList mInterfaces;
 };
 
+// Totally incomplete mini version of ChannelDispatchOperation
+class ChannelDispatchOperationAdaptor : public QDBusAbstractAdaptor
+{
+    Q_OBJECT
+    Q_CLASSINFO("D-Bus Interface", "org.freedesktop.Telepathy.ChannelDispatchOperation")
+    Q_CLASSINFO("D-Bus Introspection", ""
+"  <interface name=\"org.freedesktop.Telepathy.ChannelDispatchOperation\" >\n"
+"    <property name=\"Account\" type=\"o\" access=\"read\" />\n"
+"    <property name=\"Connection\" type=\"o\" access=\"read\" />\n"
+"    <property name=\"Channels\" type=\"a(oa{sv})\" access=\"read\" />\n"
+"    <property name=\"Interfaces\" type=\"as\" access=\"read\" />\n"
+"    <property name=\"PossibleHandlers\" type=\"as\" access=\"read\" />\n"
+"  </interface>\n"
+        "")
+
+    Q_PROPERTY(QDBusObjectPath Account READ Account)
+    Q_PROPERTY(QDBusObjectPath Connection READ Connection)
+    Q_PROPERTY(Tp::ChannelDetailsList Channels READ Channels)
+    Q_PROPERTY(QStringList Interfaces READ Interfaces)
+    Q_PROPERTY(QStringList PossibleHandlers READ PossibleHandlers)
+
+public:
+    ChannelDispatchOperationAdaptor(const QDBusObjectPath &acc, const QDBusObjectPath &conn,
+            const ChannelDetailsList &channels, const QStringList &possibleHandlers,
+            QObject *parent)
+        : QDBusAbstractAdaptor(parent), mAccount(acc), mConn(conn), mChannels(channels),
+          mPossibleHandlers(possibleHandlers)
+    {
+    }
+
+    virtual ~ChannelDispatchOperationAdaptor()
+    {
+    }
+
+public: // Properties
+    inline QDBusObjectPath Account() const
+    {
+        return mAccount;
+    }
+
+    inline QDBusObjectPath Connection() const
+    {
+        return mConn;
+    }
+
+    inline ChannelDetailsList Channels() const
+    {
+        return mChannels;
+    }
+
+    inline QStringList Interfaces() const
+    {
+        return mInterfaces;
+    }
+
+    inline QStringList PossibleHandlers() const
+    {
+        return mPossibleHandlers;
+    }
+
+private:
+    QDBusObjectPath mAccount, mConn;
+    ChannelDetailsList mChannels;
+    QStringList mInterfaces;
+    QStringList mPossibleHandlers;
+};
+
 class MyClient : public QObject,
                  public AbstractClientObserver,
                  public AbstractClientApprover,
@@ -212,7 +279,7 @@ public:
         mHandleChannelsUserActionTime = userActionTime;
         mHandleChannelsHandlerInfo = handlerInfo;
 
-        foreach (const ChannelPtr &channel, channels) {
+        Q_FOREACH (const ChannelPtr &channel, channels) {
             connect(channel.data(),
                     SIGNAL(invalidated(Tp::DBusProxy *,
                                        const QString &, const QString &)),
@@ -226,7 +293,7 @@ public:
     void addRequest(const ChannelRequestPtr &request)
     {
         mAddRequestRequest = request;
-        emit requestAdded(request);
+        Q_EMIT requestAdded(request);
     }
 
     void removeRequest(const ChannelRequestPtr &request,
@@ -235,7 +302,7 @@ public:
         mRemoveRequestRequest = request;
         mRemoveRequestErrorName = errorName;
         mRemoveRequestErrorMessage = errorMessage;
-        emit requestRemoved(request, errorName, errorMessage);
+        Q_EMIT requestRemoved(request, errorName, errorMessage);
     }
 
     AccountPtr mObserveChannelsAccount;
@@ -317,8 +384,10 @@ private:
     QString mConnPath;
 
     ClientRegistrarPtr mClientRegistrar;
-    QString mChannelRequestBusName;
+    QString mChannelDispatcherBusName;
     QString mChannelRequestPath;
+    ChannelDispatchOperationAdaptor *mCDO;
+    QString mCDOPath;
     QStringList mClientCapabilities;
     AbstractClientPtr mClientObject1;
     QString mClientObject1BusName;
@@ -369,7 +438,7 @@ void TestClient::initTestCase()
             TP_TESTS_TYPE_CONTACTS_CONNECTION,
             "account", "me@example.com",
             "protocol", "example",
-            0));
+            NULL));
     QVERIFY(mConnService != 0);
     mBaseConnService = TP_BASE_CONNECTION(mConnService);
     QVERIFY(mBaseConnService != 0);
@@ -429,9 +498,14 @@ void TestClient::initTestCase()
     mClientRegistrar = ClientRegistrar::create();
 
     QDBusConnection bus = mClientRegistrar->dbusConnection();
-    mChannelRequestBusName = QLatin1String(TELEPATHY_INTERFACE_CHANNEL_DISPATCHER);
+
+    // Fake ChannelRequest
+
+    mChannelDispatcherBusName = QLatin1String(TELEPATHY_INTERFACE_CHANNEL_DISPATCHER);
     mChannelRequestPath = QLatin1String("/org/freedesktop/Telepathy/ChannelRequest/Request1");
+
     QObject *request = new QObject(this);
+
     mUserActionTime = QDateTime::currentDateTime().toTime_t();
     new ChannelRequestAdaptor(QDBusObjectPath(mAccount->objectPath()),
             mUserActionTime,
@@ -439,8 +513,26 @@ void TestClient::initTestCase()
             QualifiedPropertyValueMapList(),
             QStringList(),
             request);
-    QVERIFY(bus.registerService(mChannelRequestBusName));
+    QVERIFY(bus.registerService(mChannelDispatcherBusName));
     QVERIFY(bus.registerObject(mChannelRequestPath, request));
+
+    // Fake ChannelDispatchOperation
+
+    mCDOPath = QLatin1String("/org/freedesktop/Telepathy/ChannelDispatchOperation/Operation1");
+
+    QObject *cdo = new QObject(this);
+
+    // Initialize this here so we can actually set it in possibleHandlers
+    mClientObject1BusName = QLatin1String("org.freedesktop.Telepathy.Client.foo");
+
+    ChannelDetailsList channelDetailsList;
+    ChannelDetails channelDetails = { QDBusObjectPath(mText1ChanPath), QVariantMap() };
+    channelDetailsList.append(channelDetails);
+
+    mCDO = new ChannelDispatchOperationAdaptor(QDBusObjectPath(mAccount->objectPath()),
+            QDBusObjectPath(mConn->objectPath()), channelDetailsList,
+            QStringList() << mClientObject1BusName, cdo);
+    QVERIFY(bus.registerObject(mCDOPath, cdo));
 }
 
 void TestClient::init()
@@ -578,7 +670,7 @@ void TestClient::testObserveChannelsCommon(const AbstractClientPtr &clientObject
     QCOMPARE(client->mObserveChannelsAccount->objectPath(), mAccount->objectPath());
     QCOMPARE(client->mObserveChannelsConnection->objectPath(), mConn->objectPath());
     QCOMPARE(client->mObserveChannelsChannels.first()->objectPath(), mText1ChanPath);
-    QCOMPARE(client->mObserveChannelsDispatchOperation->objectPath(), QString(QLatin1String("/")));
+    QVERIFY(client->mObserveChannelsDispatchOperation.isNull());
     QCOMPARE(client->mObserveChannelsRequestsSatisfied.first()->objectPath(), mChannelRequestPath);
 }
 
@@ -600,21 +692,25 @@ void TestClient::testAddDispatchOperation()
     connect(client,
             SIGNAL(addDispatchOperationFinished()),
             SLOT(expectSignalEmission()));
-    ChannelDetailsList channelDetailsList;
-    ChannelDetails channelDetails = { QDBusObjectPath(mText1ChanPath), QVariantMap() };
-    channelDetailsList.append(channelDetails);
+
     QVariantMap dispatchOperationProperties;
-    dispatchOperationProperties.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_DISPATCH_OPERATION ".Connection"),
+    dispatchOperationProperties.insert(
+            QLatin1String(TELEPATHY_INTERFACE_CHANNEL_DISPATCH_OPERATION ".Connection"),
             QVariant::fromValue(QDBusObjectPath(mConn->objectPath())));
-    dispatchOperationProperties.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_DISPATCH_OPERATION ".Account"),
+    dispatchOperationProperties.insert(
+            QLatin1String(TELEPATHY_INTERFACE_CHANNEL_DISPATCH_OPERATION ".Account"),
             QVariant::fromValue(QDBusObjectPath(mAccount->objectPath())));
-    approverIface->AddDispatchOperation(channelDetailsList,
-            QDBusObjectPath(QLatin1String("/")),
+    dispatchOperationProperties.insert(
+            QLatin1String(TELEPATHY_INTERFACE_CHANNEL_DISPATCH_OPERATION ".PossibleHandlers"),
+            QVariant::fromValue(ObjectPathList() << QDBusObjectPath(mClientObject1Path)
+                << QDBusObjectPath(mClientObject2Path)));
+
+    approverIface->AddDispatchOperation(mCDO->Channels(), QDBusObjectPath(mCDOPath),
             dispatchOperationProperties);
     QCOMPARE(mLoop->exec(), 0);
 
     QCOMPARE(client->mAddDispatchOperationChannels.first()->objectPath(), mText1ChanPath);
-    QCOMPARE(client->mAddDispatchOperationDispatchOperation->objectPath(), QString(QLatin1String("/")));
+    QCOMPARE(client->mAddDispatchOperationDispatchOperation->objectPath(), mCDOPath);
 }
 
 void TestClient::testHandleChannels()
