@@ -10,6 +10,7 @@
 #include <TelepathyQt4/PendingReady>
 #include <TelepathyQt4/Debug>
 
+#include <telepathy-glib/dbus.h>
 #include <telepathy-glib/debug.h>
 
 #include <tests/lib/glib/contacts-conn.h>
@@ -35,6 +36,7 @@ private Q_SLOTS:
     void initTestCase();
     void init();
 
+    void testBasics();
     void testSimplePresence();
 
     void cleanup();
@@ -92,6 +94,11 @@ void TestConnBasics::initTestCase()
     g_set_prgname("conn-basics");
     tp_debug_set_flags("all");
     dbus_g_bus_get(DBUS_BUS_STARTER, 0);
+}
+
+void TestConnBasics::init()
+{
+    initImpl();
 
     gchar *name;
     gchar *connPath;
@@ -101,7 +108,7 @@ void TestConnBasics::initTestCase()
             TP_TESTS_TYPE_CONTACTS_CONNECTION,
             "account", "me@example.com",
             "protocol", "contacts",
-            0));
+            NULL));
     QVERIFY(mConnService != 0);
     QVERIFY(tp_base_connection_register(TP_BASE_CONNECTION(mConnService),
                 "contacts", &name, &connPath, &error));
@@ -115,11 +122,6 @@ void TestConnBasics::initTestCase()
 
     g_free(name);
     g_free(connPath);
-}
-
-void TestConnBasics::init()
-{
-    initImpl();
 
     mConn = Connection::create(mConnName, mConnPath);
     QCOMPARE(mConn->isReady(), false);
@@ -147,8 +149,16 @@ void TestConnBasics::init()
     }
 }
 
+void TestConnBasics::testBasics()
+{
+    QCOMPARE(static_cast<uint>(mConn->statusReason()),
+            static_cast<uint>(ConnectionStatusReasonRequested));
+}
+
 void TestConnBasics::testSimplePresence()
 {
+    qDebug() << "Making SimplePresence ready";
+
     Features features = Features() << Connection::FeatureSimplePresence;
     QCOMPARE(mConn->isReady(features), false);
     QVERIFY(connect(mConn->becomeReady(features),
@@ -157,25 +167,101 @@ void TestConnBasics::testSimplePresence()
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mConn->isReady(features), true);
 
+    qDebug() << "SimplePresence ready";
     qDebug() << "mConn->status:" << mConn->status();
+
+    const QStringList canSetNames = QStringList()
+        << QLatin1String("available")
+        << QLatin1String("busy")
+        << QLatin1String("away");
+
+    const QStringList cantSetNames = QStringList()
+        << QLatin1String("offline")
+        << QLatin1String("unknown")
+        << QLatin1String("error");
+
+    const QStringList expectedNames = canSetNames + cantSetNames;
+
+    const ConnectionPresenceType expectedTypes[] = {
+        ConnectionPresenceTypeAvailable,
+        ConnectionPresenceTypeBusy,
+        ConnectionPresenceTypeAway,
+        ConnectionPresenceTypeOffline,
+        ConnectionPresenceTypeUnknown,
+        ConnectionPresenceTypeError
+    };
+
+    SimpleStatusSpecMap statuses = mConn->allowedPresenceStatuses();
+    Q_FOREACH (QString name, statuses.keys()) {
+        QVERIFY(expectedNames.contains(name));
+
+        if (canSetNames.contains(name)) {
+            QVERIFY(statuses[name].maySetOnSelf);
+            QVERIFY(statuses[name].canHaveMessage);
+        } else {
+            QVERIFY(cantSetNames.contains(name));
+            QVERIFY(!statuses[name].maySetOnSelf);
+            QVERIFY(!statuses[name].canHaveMessage);
+        }
+
+        QCOMPARE(statuses[name].type,
+                 static_cast<uint>(expectedTypes[expectedNames.indexOf(name)]));
+    }
 }
 
 void TestConnBasics::cleanup()
 {
     if (mConn) {
-        // Disconnect and wait for the readiness change
-        QVERIFY(connect(mConn->requestDisconnect(),
-                        SIGNAL(finished(Tp::PendingOperation*)),
-                        SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
-        QCOMPARE(mLoop->exec(), 0);
+        QVERIFY(mConn->isValid());
 
-        if (mConn->isValid()) {
-            QVERIFY(connect(mConn.data(),
-                            SIGNAL(invalidated(Tp::DBusProxy *,
-                                               const QString &, const QString &)),
-                            SLOT(expectConnInvalidated())));
-            QCOMPARE(mLoop->exec(), 0);
-        }
+        GHashTable *details = tp_asv_new(
+                "debug-message", G_TYPE_STRING, "woo i'm going doooooown",
+                "x-tpqt4-test-rgba-herring-color", G_TYPE_UINT, 0xff0000ffU,
+                NULL
+                );
+
+        // Disconnect and wait for invalidation
+        tp_base_connection_disconnect_with_dbus_error(TP_BASE_CONNECTION(mConnService),
+                TELEPATHY_ERROR_CANCELLED,
+                details,
+                TP_CONNECTION_STATUS_REASON_REQUESTED);
+
+        g_hash_table_destroy(details);
+
+        QVERIFY(connect(mConn.data(),
+                    SIGNAL(invalidated(Tp::DBusProxy *,
+                            const QString &, const QString &)),
+                    SLOT(expectConnInvalidated())));
+        QCOMPARE(mLoop->exec(), 0);
+        QVERIFY(!mConn->isValid());
+
+        // Check that we got the connection error details
+        QCOMPARE(static_cast<uint>(mConn->statusReason()),
+                 static_cast<uint>(ConnectionStatusReasonRequested));
+
+        QVERIFY(mConn->errorDetails().isValid());
+
+        QVERIFY(mConn->errorDetails().hasDebugMessage());
+        QCOMPARE(mConn->errorDetails().debugMessage(), QLatin1String("woo i'm going doooooown"));
+
+#if 0
+        // Not yet there
+        QVERIFY(!mConn->errorDetails().hasExpectedHostname());
+        QVERIFY(!mConn->errorDetails().hasCertificateHostname());
+#endif
+
+        QVERIFY(mConn->errorDetails().allDetails().contains(
+                    QLatin1String("x-tpqt4-test-rgba-herring-color")));
+        QCOMPARE(qdbus_cast<uint>(mConn->errorDetails().allDetails().value(
+                        QLatin1String("x-tpqt4-test-rgba-herring-color"))),
+                0xff0000ffU);
+
+        mConn.reset();
+    }
+
+    if (mConnService != 0) {
+        g_object_unref(mConnService);
+        mConnService = 0;
     }
 
     cleanupImpl();
@@ -183,11 +269,6 @@ void TestConnBasics::cleanup()
 
 void TestConnBasics::cleanupTestCase()
 {
-    if (mConnService != 0) {
-        g_object_unref(mConnService);
-        mConnService = 0;
-    }
-
     cleanupTestCaseImpl();
 }
 
