@@ -100,7 +100,7 @@ void TestContactsInfo::initTestCase()
             TP_TESTS_TYPE_CONTACTS_CONNECTION,
             "account", "me@example.com",
             "protocol", "foo",
-            0));
+            NULL));
     QVERIFY(mConnService != 0);
     QVERIFY(tp_base_connection_register(TP_BASE_CONNECTION(mConnService),
                 "foo", &name, &connPath, &error));
@@ -214,7 +214,7 @@ void TestContactsInfo::testInfo()
     QCOMPARE(contactBar->info()[0].fieldName, QLatin1String("n"));
     QCOMPARE(contactBar->info()[0].fieldValue[0], QLatin1String("Bar"));
 
-    foreach (const ContactPtr &contact, mContacts) {
+    Q_FOREACH (const ContactPtr &contact, mContacts) {
         PendingOperation *op = contact->refreshInfo();
         QVERIFY(connect(op,
                     SIGNAL(finished(Tp::PendingOperation*)),
