@@ -139,7 +139,7 @@ void TestContacts::initTestCase()
             TP_TESTS_TYPE_CONTACTS_CONNECTION,
             "account", "me@example.com",
             "protocol", "simple",
-            0));
+            NULL));
     QVERIFY(mConnService != 0);
     QVERIFY(tp_base_connection_register(TP_BASE_CONNECTION(mConnService), "contacts", &name,
                 &connPath, &error));
@@ -415,7 +415,7 @@ void TestContacts::testForIdentifiers()
     processDBusQueue(mConn.data());
 
     // Check that their handles are in fact released
-    foreach (uint handle, saveHandles) {
+    Q_FOREACH (uint handle, saveHandles) {
         QVERIFY(!tp_handle_is_valid(serviceRepo, handle, NULL));
     }
 }
@@ -761,7 +761,7 @@ void TestContacts::testSelfContactFallback()
             TP_TESTS_TYPE_SIMPLE_CONNECTION,
             "account", "me@example.com",
             "protocol", "simple",
-            0));
+            NULL));
     QVERIFY(connService != 0);
     QVERIFY(tp_base_connection_register(TP_BASE_CONNECTION(connService), "simple", &name,
                 &connPath, &error));
@@ -793,6 +793,19 @@ void TestContacts::testSelfContactFallback()
     QCOMPARE(selfContact->alias(), QString(QLatin1String("me@example.com")));
     QVERIFY(!selfContact->isAvatarTokenKnown());
     QCOMPARE(selfContact->presenceStatus(), QString(QLatin1String("unknown")));
+
+    tp_tests_simple_connection_inject_disconnect(connService);
+
+    if (conn->isValid()) {
+        QVERIFY(connect(conn.data(),
+                    SIGNAL(invalidated(Tp::DBusProxy *,
+                            const QString &, const QString &)),
+                    mLoop,
+                    SLOT(quit())));
+        QCOMPARE(mLoop->exec(), 0);
+    }
+
+    g_object_unref(connService);
 }
 
 void TestContacts::cleanup()
