@@ -18,12 +18,12 @@
 
 using namespace Tp;
 
-class TestStreamedMediaChan : public Test
+class TestStreamedMediaChanCall : public Test
 {
     Q_OBJECT
 
 public:
-    TestStreamedMediaChan(QObject *parent = 0)
+    TestStreamedMediaChanCall(QObject *parent = 0)
         : Test(parent), mConnService(0)
     { }
 
@@ -31,6 +31,7 @@ protected Q_SLOTS:
     void expectRequestContactsFinished(Tp::PendingOperation *);
     void expectCreateChannelFinished(Tp::PendingOperation *);
     void expectRequestContentFinished(Tp::PendingOperation *);
+    void expectSuccessfulRequestReceiving(Tp::PendingOperation *);
     void onLocalSendingStateChanged(Tp::MediaStream::SendingState);
     void onRemoteSendingStateChanged(const QHash<Tp::ContactPtr, Tp::MediaStream::SendingState> &);
     void onChanInvalidated(Tp::DBusProxy *,
@@ -61,12 +62,20 @@ private:
     QList<ContactPtr> mRequestContactsReturn;
     MediaContentPtr mRequestContentReturn;
     MediaStream::SendingState mLSSCReturn;
-    QHash<ContactPtr, MediaStream::SendingState> mRSSCReturn;
     QQueue<uint> mLocalHoldStates;
     QQueue<uint> mLocalHoldStateReasons;
+
+    // Remote sending state changed state-machine
+    enum {
+        RSSCStateInitial,
+        RSSCStatePendingSend,
+        RSSCStateSending,
+        RSSCStateDone
+    } mRSSCState;
+    int mSuccessfulRequestReceivings;
 };
 
-void TestStreamedMediaChan::expectRequestContactsFinished(PendingOperation *op)
+void TestStreamedMediaChanCall::expectRequestContactsFinished(PendingOperation *op)
 {
     if (!op->isFinished()) {
         qWarning() << "unfinished";
@@ -92,7 +101,7 @@ void TestStreamedMediaChan::expectRequestContactsFinished(PendingOperation *op)
     mLoop->exit(0);
 }
 
-void TestStreamedMediaChan::expectCreateChannelFinished(PendingOperation* op)
+void TestStreamedMediaChanCall::expectCreateChannelFinished(PendingOperation* op)
 {
     if (!op->isFinished()) {
         qWarning() << "unfinished";
@@ -114,11 +123,11 @@ void TestStreamedMediaChan::expectCreateChannelFinished(PendingOperation* op)
     }
 
     PendingChannel *pc = qobject_cast<PendingChannel*>(op);
-    mChan = StreamedMediaChannelPtr::dynamicCast(pc->channel());
+    mChan = StreamedMediaChannelPtr::qObjectCast(pc->channel());
     mLoop->exit(0);
 }
 
-void TestStreamedMediaChan::expectRequestContentFinished(PendingOperation *op)
+void TestStreamedMediaChanCall::expectRequestContentFinished(PendingOperation *op)
 {
     if (!op->isFinished()) {
         qWarning() << "unfinished";
@@ -144,7 +153,7 @@ void TestStreamedMediaChan::expectRequestContentFinished(PendingOperation *op)
     mLoop->exit(0);
 }
 
-void TestStreamedMediaChan::onLocalSendingStateChanged(
+void TestStreamedMediaChanCall::onLocalSendingStateChanged(
         const MediaStream::SendingState state)
 {
     qDebug() << "local sending state changed";
@@ -152,25 +161,80 @@ void TestStreamedMediaChan::onLocalSendingStateChanged(
     mLoop->exit(0);
 }
 
-void TestStreamedMediaChan::onRemoteSendingStateChanged(
-        const QHash<Tp::ContactPtr, Tp::MediaStream::SendingState> &states)
+void TestStreamedMediaChanCall::expectSuccessfulRequestReceiving(PendingOperation *op)
 {
-    qDebug() << "remote sending state changed";
-    mRSSCReturn = states;
-    mLoop->exit(0);
+    if (!op->isFinished()) {
+        qWarning() << "unfinished";
+        mLoop->exit(1);
+        return;
+    }
+
+    if (op->isError()) {
+        qWarning().nospace() << op->errorName()
+            << ": " << op->errorMessage();
+        mLoop->exit(2);
+        return;
+    }
+
+    if (!op->isValid()) {
+        qWarning() << "inconsistent results";
+        mLoop->exit(3);
+        return;
+    }
+
+    if (++mSuccessfulRequestReceivings == 2 && mRSSCState == RSSCStateDone) {
+        mLoop->exit(0);
+    }
 }
 
-void TestStreamedMediaChan::onChanInvalidated(Tp::DBusProxy *proxy,
+void TestStreamedMediaChanCall::onRemoteSendingStateChanged(
+        const QHash<Tp::ContactPtr, Tp::MediaStream::SendingState> &states)
+{
+    // There should be no further events
+    QVERIFY(mRSSCState != RSSCStateDone);
+
+    QCOMPARE(states.size(), 1);
+    ContactPtr otherContact = states.keys().first();
+
+    MediaContentPtr content = mChan->contentsForType(Tp::MediaStreamTypeVideo).first();
+    QVERIFY(content);
+
+    MediaStreamPtr stream = content->streams().first();
+    QVERIFY(stream);
+
+    if (mRSSCState == RSSCStateInitial) {
+        QCOMPARE(states[otherContact], MediaStream::SendingStatePendingSend);
+        mRSSCState = RSSCStatePendingSend;
+    } else if (mRSSCState == RSSCStatePendingSend) {
+        QCOMPARE(states[otherContact], MediaStream::SendingStateSending);
+        mRSSCState = RSSCStateSending;
+
+        QVERIFY(connect(stream->requestReceiving(otherContact, false),
+                    SIGNAL(finished(Tp::PendingOperation*)),
+                    SLOT(expectSuccessfulRequestReceiving(Tp::PendingOperation*))));
+    } else if (mRSSCState == RSSCStateSending) {
+        QCOMPARE(states[otherContact], MediaStream::SendingStateNone);
+        mRSSCState = RSSCStateDone;
+
+        if (mSuccessfulRequestReceivings == 2) {
+            mLoop->exit(0);
+        }
+    }
+
+    qDebug() << "remote sending state changed to" << states[otherContact];
+}
+
+void TestStreamedMediaChanCall::onChanInvalidated(Tp::DBusProxy *proxy,
         const QString &errorName, const QString &errorMessage)
 {
     qDebug() << "chan invalidated:" << errorName << "-" << errorMessage;
     mLoop->exit(0);
 }
 
-void TestStreamedMediaChan::onNewChannels(const Tp::ChannelDetailsList &channels)
+void TestStreamedMediaChanCall::onNewChannels(const Tp::ChannelDetailsList &channels)
 {
     qDebug() << "new channels";
-    foreach (const Tp::ChannelDetails &details, channels) {
+    Q_FOREACH (const Tp::ChannelDetails &details, channels) {
         QString channelType = details.properties.value(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".ChannelType")).toString();
         bool requested = details.properties.value(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".Requested")).toBool();
         qDebug() << " channelType:" << channelType;
@@ -186,7 +250,7 @@ void TestStreamedMediaChan::onNewChannels(const Tp::ChannelDetailsList &channels
     }
 }
 
-void TestStreamedMediaChan::onLocalHoldStateChanged(Tp::LocalHoldState localHoldState,
+void TestStreamedMediaChanCall::onLocalHoldStateChanged(Tp::LocalHoldState localHoldState,
         Tp::LocalHoldStateReason localHoldStateReason)
 {
     mLocalHoldStates.append(localHoldState);
@@ -194,7 +258,7 @@ void TestStreamedMediaChan::onLocalHoldStateChanged(Tp::LocalHoldState localHold
     mLoop->exit(0);
 }
 
-void TestStreamedMediaChan::initTestCase()
+void TestStreamedMediaChanCall::initTestCase()
 {
     initTestCaseImpl();
 
@@ -211,7 +275,8 @@ void TestStreamedMediaChan::initTestCase()
             EXAMPLE_TYPE_CALL_CONNECTION,
             "account", "me@example.com",
             "protocol", "example",
-            0));
+            "simulation-delay", 1,
+            NULL));
     QVERIFY(mConnService != 0);
     QVERIFY(tp_base_connection_register(TP_BASE_CONNECTION(mConnService),
                 "example", &name, &connPath, &error));
@@ -238,20 +303,21 @@ void TestStreamedMediaChan::initTestCase()
              static_cast<uint>(Connection::StatusConnected));
 }
 
-void TestStreamedMediaChan::init()
+void TestStreamedMediaChanCall::init()
 {
     initImpl();
 
     mRequestContactsReturn.clear();
     mRequestContentReturn.reset();
     mLSSCReturn = (MediaStream::SendingState) -1;
-    mRSSCReturn.clear();
     mLocalHoldStates.clear();
     mLocalHoldStateReasons.clear();
 }
 
-void TestStreamedMediaChan::testOutgoingCall()
+void TestStreamedMediaChanCall::testOutgoingCall()
 {
+    qDebug() << "requesting contact for alice";
+
     QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(QStringList() << QLatin1String("alice")),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectRequestContactsFinished(Tp::PendingOperation*))));
@@ -260,6 +326,8 @@ void TestStreamedMediaChan::testOutgoingCall()
     ContactPtr otherContact = mRequestContactsReturn.first();
     QVERIFY(otherContact);
 
+    qDebug() << "creating the channel";
+
     QVariantMap request;
     request.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".ChannelType"),
                    QLatin1String(TP_FUTURE_INTERFACE_CHANNEL_TYPE_CALL));
@@ -275,6 +343,8 @@ void TestStreamedMediaChan::testOutgoingCall()
     QCOMPARE(mLoop->exec(), 0);
     QVERIFY(mChan);
 
+    qDebug() << "making the channel ready";
+
     QVERIFY(connect(mChan->becomeReady(StreamedMediaChannel::FeatureContents),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
@@ -291,6 +361,8 @@ void TestStreamedMediaChan::testOutgoingCall()
     QVERIFY(mChan->groupContacts().contains(mConn->selfContact()));
     QVERIFY(mChan->groupContacts().contains(otherContact));
 
+    qDebug() << "calling requestContent with a bad type";
+
     // RequestContent with bad type must fail
     QVERIFY(connect(mChan->requestContent(QLatin1String("content1"), (Tp::MediaStreamType) -1),
                     SIGNAL(finished(Tp::PendingOperation*)),
@@ -298,6 +370,8 @@ void TestStreamedMediaChan::testOutgoingCall()
     QCOMPARE(mLoop->exec(), 2);
     QVERIFY(!mRequestContentReturn);
 
+    qDebug() << "calling requestContent with Video";
+
     // Request video content
     QVERIFY(connect(mChan->requestContent(QLatin1String("content2"), Tp::MediaStreamTypeVideo),
                     SIGNAL(finished(Tp::PendingOperation*)),
@@ -329,6 +403,8 @@ void TestStreamedMediaChan::testOutgoingCall()
     MediaStreamPtr stream = content->streams().first();
     QVERIFY(content);
 
+    qDebug() << "stopping sending";
+
     QVERIFY(connect(stream.data(),
                     SIGNAL(localSendingStateChanged(Tp::MediaStream::SendingState)),
                     SLOT(onLocalSendingStateChanged(Tp::MediaStream::SendingState))));
@@ -336,12 +412,26 @@ void TestStreamedMediaChan::testOutgoingCall()
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
-    while (mLSSCReturn == (MediaStream::SendingState) -1) {
-        // wait sending state change
+
+    qDebug() << "stopping receiving";
+
+    QVERIFY(connect(stream->requestReceiving(otherContact, false),
+                    SIGNAL(finished(Tp::PendingOperation*)),
+                    SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
+    QCOMPARE(mLoop->exec(), 0);
+
+    qDebug() << "waiting until we're not sending and not receiving";
+
+    while (stream->localSendingState() != MediaStream::SendingStateNone
+            || stream->remoteSendingState(otherContact) != MediaStream::SendingStateNone) {
+        qDebug() << "re-entering mainloop to wait for local and remote SSC -> None";
+        // wait local and remote sending state change
         QCOMPARE(mLoop->exec(), 0);
     }
     QCOMPARE(mLSSCReturn, MediaStream::SendingStateNone);
 
+    qDebug() << "re-enabling sending";
+
     mLSSCReturn = (MediaStream::SendingState) -1;
 
     QVERIFY(connect(stream->requestSending(true),
@@ -349,46 +439,35 @@ void TestStreamedMediaChan::testOutgoingCall()
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     while (mLSSCReturn == (MediaStream::SendingState) -1) {
+        qDebug() << "re-entering mainloop to wait for SSC -> Sending";
         // wait sending state change
         QCOMPARE(mLoop->exec(), 0);
     }
     QCOMPARE(mLSSCReturn, MediaStream::SendingStateSending);
 
+    qDebug() << "flushing D-Bus events";
+    processDBusQueue(mChan.data());
+
+    qDebug() << "enabling receiving";
+
+    mRSSCState = RSSCStateInitial;
+    mSuccessfulRequestReceivings = 0;
+
     // test content receiving changed
     QVERIFY(connect(stream.data(),
                     SIGNAL(remoteSendingStateChanged(const QHash<Tp::ContactPtr, Tp::MediaStream::SendingState> &)),
                     SLOT(onRemoteSendingStateChanged(const QHash<Tp::ContactPtr, Tp::MediaStream::SendingState> &))));
     QVERIFY(connect(stream->requestReceiving(otherContact, true),
                     SIGNAL(finished(Tp::PendingOperation*)),
-                    SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
-    while (mRSSCReturn.isEmpty()) {
-        // wait remote sending state change
-        QCOMPARE(mLoop->exec(), 0);
+                    SLOT(expectSuccessfulRequestReceiving(Tp::PendingOperation*))));
+    while (mSuccessfulRequestReceivings != 2 || mRSSCState != RSSCStateDone) {
+        mLoop->processEvents();
     }
-    QCOMPARE(mRSSCReturn.value(otherContact), MediaStream::SendingStatePendingSend);
 
-    mRSSCReturn.clear();
-
-    while (mRSSCReturn.isEmpty()) {
-        // wait remote sending state change
-        QCOMPARE(mLoop->exec(), 0);
-    }
-    QCOMPARE(mRSSCReturn.value(otherContact), MediaStream::SendingStateSending);
-
-    mRSSCReturn.clear();
-
-    QVERIFY(connect(stream->requestReceiving(otherContact, false),
-                    SIGNAL(finished(Tp::PendingOperation*)),
-                    SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
-    while (mRSSCReturn.isEmpty()) {
-        // wait remote sending state change
-        QCOMPARE(mLoop->exec(), 0);
-    }
-    QCOMPARE(mRSSCReturn.value(otherContact), MediaStream::SendingStateNone);
+    QCOMPARE(static_cast<uint>(mRSSCState), static_cast<uint>(RSSCStateDone));
 }
 
-void TestStreamedMediaChan::testHold()
+void TestStreamedMediaChanCall::testHold()
 {
     QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(QStringList() << QLatin1String("bob")),
                     SIGNAL(finished(Tp::PendingOperation*)),
@@ -459,7 +538,7 @@ void TestStreamedMediaChan::testHold()
     QCOMPARE(static_cast<uint>(mChan->localHoldStateReason()), static_cast<uint>(LocalHoldStateReasonRequested));
 }
 
-void TestStreamedMediaChan::testHoldNoUnhold()
+void TestStreamedMediaChanCall::testHoldNoUnhold()
 {
     QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(QStringList() << QLatin1String("bob (no unhold)")),
                     SIGNAL(finished(Tp::PendingOperation*)),
@@ -497,13 +576,11 @@ void TestStreamedMediaChan::testHoldNoUnhold()
                     SIGNAL(localHoldStateChanged(Tp::LocalHoldState, Tp::LocalHoldStateReason)),
                     SLOT(onLocalHoldStateChanged(Tp::LocalHoldState, Tp::LocalHoldStateReason))));
     // Request hold
-    QVERIFY(connect(mChan->requestHold(true),
-                    SIGNAL(finished(Tp::PendingOperation*)),
-                    SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
-    while (mLocalHoldStates.size() != 2) {
-        QCOMPARE(mLoop->exec(), 0);
+    QPointer<PendingOperation> holdOp = mChan->requestHold(true);
+    while (mLocalHoldStates.size() != 2 || (holdOp && !holdOp->isFinished())) {
+        mLoop->processEvents();
     }
+    QCOMPARE(!holdOp || holdOp->isValid(), true);
     QCOMPARE(mLocalHoldStates.first(), static_cast<uint>(LocalHoldStatePendingHold));
     QCOMPARE(mLocalHoldStateReasons.first(), static_cast<uint>(LocalHoldStateReasonRequested));
     QCOMPARE(mLocalHoldStates.last(), static_cast<uint>(LocalHoldStateHeld));
@@ -525,7 +602,7 @@ void TestStreamedMediaChan::testHoldNoUnhold()
     QCOMPARE(static_cast<uint>(mChan->localHoldStateReason()), static_cast<uint>(LocalHoldStateReasonRequested));
 }
 
-void TestStreamedMediaChan::testHoldInabilityUnhold()
+void TestStreamedMediaChanCall::testHoldInabilityUnhold()
 {
     QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(QStringList() << QLatin1String("bob (inability to unhold)")),
                     SIGNAL(finished(Tp::PendingOperation*)),
@@ -609,13 +686,13 @@ void TestStreamedMediaChan::testHoldInabilityUnhold()
     QCOMPARE(mChan->localHoldStateReason(), LocalHoldStateReasonRequested);
 }
 
-void TestStreamedMediaChan::cleanup()
+void TestStreamedMediaChanCall::cleanup()
 {
     mChan.reset();
     cleanupImpl();
 }
 
-void TestStreamedMediaChan::cleanupTestCase()
+void TestStreamedMediaChanCall::cleanupTestCase()
 {
     if (mConn) {
         QVERIFY(connect(mConn->requestDisconnect(),
@@ -636,5 +713,5 @@ void TestStreamedMediaChan::cleanupTestCase()
     cleanupTestCaseImpl();
 }
 
-QTEST_MAIN(TestStreamedMediaChan)
+QTEST_MAIN(TestStreamedMediaChanCall)
 #include "_gen/streamed-media-chan-call.cpp.moc.hpp"
