@@ -1,4 +1,5 @@
 #include <QtCore/QDebug>
+#include <QtCore/QPointer>
 #include <QtTest/QtTest>
 
 #include <TelepathyQt4/Connection>
@@ -29,12 +30,25 @@ protected Q_SLOTS:
     void expectRequestContactsFinished(Tp::PendingOperation *);
     void expectCreateChannelFinished(Tp::PendingOperation *);
     void expectRequestStreamsFinished(Tp::PendingOperation *);
+    void expectBusyRequestStreamsFinished(Tp::PendingOperation *);
+
+    // Special event handlers for the OutgoingCall state-machine
+    void expectOutgoingRequestStreamsFinished(Tp::PendingOperation *);
+
+    void onOutgoingGroupMembersChanged(
+            const Tp::Contacts &,
+            const Tp::Contacts &,
+            const Tp::Contacts &,
+            const Tp::Contacts &,
+            const Tp::Channel::GroupMemberChangeDetails &);
+
     void onGroupMembersChanged(
             const Tp::Contacts &,
             const Tp::Contacts &,
             const Tp::Contacts &,
             const Tp::Contacts &,
             const Tp::Channel::GroupMemberChangeDetails &);
+
     void onStreamRemoved(const Tp::MediaStreamPtr &);
     void onStreamDirectionChanged(const Tp::MediaStreamPtr &,
             Tp::MediaStreamDirection,
@@ -43,6 +57,20 @@ protected Q_SLOTS:
             Tp::MediaStreamState);
     void onChanInvalidated(Tp::DBusProxy *,
             const QString &, const QString &);
+
+    // Special event handlers for the OutgoingCallTerminate state-machine
+    void expectTerminateRequestStreamsFinished(Tp::PendingOperation *);
+
+    void onTerminateGroupMembersChanged(
+            const Tp::Contacts &,
+            const Tp::Contacts &,
+            const Tp::Contacts &,
+            const Tp::Contacts &,
+            const Tp::Channel::GroupMemberChangeDetails &);
+
+    void onTerminateChanInvalidated(Tp::DBusProxy *,
+            const QString &, const QString &);
+
     void onNewChannels(const Tp::ChannelDetailsList &);
     void onLocalHoldStateChanged(Tp::LocalHoldState,
             Tp::LocalHoldStateReason);
@@ -87,6 +115,25 @@ private:
     Tp::MediaStreamState mSSCStateReturn;
     QQueue<uint> mLocalHoldStates;
     QQueue<uint> mLocalHoldStateReasons;
+
+    // state machine for the OutgoingCall test-case
+    enum {
+        OutgoingStateInitial,
+        OutgoingStateRequested,
+        OutgoingStateRinging,
+        OutgoingStateDone
+    } mOutgoingState;
+    bool mOutgoingGotRequestStreamsFinished;
+    bool mOutgoingAudioDone;
+
+    // state machine for the OutgoingCallTerminate test-case
+    enum {
+        TerminateStateInitial,
+        TerminateStateRequested,
+        TerminateStateRinging,
+        TerminateStateAnswered,
+        TerminateStateTerminated
+    } mTerminateState;
 };
 
 void TestStreamedMediaChan::expectRequestContactsFinished(PendingOperation *op)
@@ -110,6 +157,8 @@ void TestStreamedMediaChan::expectRequestContactsFinished(PendingOperation *op)
         return;
     }
 
+    qDebug() << "contacts requested successfully";
+
     PendingContacts *pc = qobject_cast<PendingContacts*>(op);
     mRequestContactsReturn = pc->contacts();
     mLoop->exit(0);
@@ -136,13 +185,17 @@ void TestStreamedMediaChan::expectCreateChannelFinished(PendingOperation* op)
         return;
     }
 
+    qDebug() << "channel created successfully";
+
     PendingChannel *pc = qobject_cast<PendingChannel*>(op);
-    mChan = StreamedMediaChannelPtr::dynamicCast(pc->channel());
+    mChan = StreamedMediaChannelPtr::qObjectCast(pc->channel());
     mLoop->exit(0);
 }
 
 void TestStreamedMediaChan::expectRequestStreamsFinished(PendingOperation *op)
 {
+    mRequestStreamsReturn.clear();
+
     if (!op->isFinished()) {
         qWarning() << "unfinished";
         mLoop->exit(1);
@@ -162,11 +215,137 @@ void TestStreamedMediaChan::expectRequestStreamsFinished(PendingOperation *op)
         return;
     }
 
+    qDebug() << "request streams finished successfully";
+
     PendingMediaStreams *pms = qobject_cast<PendingMediaStreams*>(op);
     mRequestStreamsReturn = pms->streams();
     mLoop->exit(0);
 }
 
+void TestStreamedMediaChan::expectBusyRequestStreamsFinished(PendingOperation *op)
+{
+    if (!op->isFinished()) {
+        qWarning() << "unfinished";
+        mLoop->exit(1);
+        return;
+    }
+
+    if (op->isError()) {
+        // The service signaled busy even before tp-qt4 finished introspection.
+        // FIXME: should the error be something else, actually? Such as, perchance,
+        // org.freedesktop.Telepathy.Error.Busy? (fd.o #29757).
+        QCOMPARE(op->errorName(), QLatin1String("org.freedesktop.Telepathy.Error.Cancelled"));
+        qDebug() << "request streams finished already busy";
+        mLoop->exit(0);
+        return;
+    }
+
+    qDebug() << "request streams finished successfully";
+
+    PendingMediaStreams *pms = qobject_cast<PendingMediaStreams*>(op);
+    mRequestStreamsReturn = pms->streams();
+    mLoop->exit(0);
+}
+
+void TestStreamedMediaChan::expectOutgoingRequestStreamsFinished(PendingOperation *op)
+{
+    QVERIFY(op->isFinished());
+    QVERIFY(!op->isError());
+    QVERIFY(op->isValid());
+
+    PendingMediaStreams *pms = qobject_cast<PendingMediaStreams*>(op);
+    mRequestStreamsReturn = pms->streams();
+
+    ContactPtr otherContact = mRequestContactsReturn.first();
+    QVERIFY(otherContact);
+
+    QCOMPARE(mRequestContactsReturn.size(), 1);
+    MediaStreamPtr stream = mRequestStreamsReturn.first();
+    QCOMPARE(stream->contact(), otherContact);
+    QCOMPARE(stream->type(), Tp::MediaStreamTypeAudio);
+
+    // These checks can't work reliably, unless we add some complex backdoors to the test service,
+    // to only start changing state / direction when we explicitly tell it so (not automatically
+    // when we have requested the stream)
+    // QCOMPARE(stream->state(), Tp::MediaStreamStateDisconnected);
+    // QCOMPARE(stream->direction(), Tp::MediaStreamDirectionBidirectional);
+
+    QCOMPARE(mChan->streams().size(), 1);
+    QVERIFY(mChan->streams().contains(stream));
+
+    qDebug() << "stream requested successfully";
+
+    // Only advance to Requested if the remote moving to RP hasn't already advanced it to Ringing or
+    // even Answered - tbf it seems the StreamedMediaChannel semantics are quite hard for
+    // application code to get right because the events can happen in whichever order. Should this
+    // be considered a bug by itself? It'd probably be pretty hard to fix so I hope not :D
+    if (mOutgoingState == OutgoingStateInitial) {
+        mOutgoingState = OutgoingStateRequested;
+    }
+
+    if (mOutgoingState == OutgoingStateDone) {
+        // finished later than the membersChanged() - exit mainloop now
+        mLoop->exit(0);
+    } else {
+        // finished earlier than membersChanged() - it will exit
+        mOutgoingGotRequestStreamsFinished = true;
+    }
+}
+
+void TestStreamedMediaChan::onOutgoingGroupMembersChanged(
+        const Contacts &groupMembersAdded,
+        const Contacts &groupLocalPendingMembersAdded,
+        const Contacts &groupRemotePendingMembersAdded,
+        const Contacts &groupMembersRemoved,
+        const Channel::GroupMemberChangeDetails &details)
+{
+    // At this point, mRequestContactsReturn should still contain the contact we requested the
+    // stream for
+    ContactPtr otherContact = mRequestContactsReturn.first();
+
+    if (mOutgoingState == OutgoingStateInitial || mOutgoingState == OutgoingStateRequested) {
+        // The target should have become remote pending now
+        QVERIFY(groupMembersAdded.isEmpty());
+        QVERIFY(groupLocalPendingMembersAdded.isEmpty());
+        QCOMPARE(groupRemotePendingMembersAdded.size(), 1);
+        QVERIFY(groupMembersRemoved.isEmpty());
+
+        QVERIFY(mChan->groupRemotePendingContacts().contains(otherContact));
+        QCOMPARE(mChan->awaitingRemoteAnswer(), true);
+
+        qDebug() << "call now ringing";
+
+        mOutgoingState = OutgoingStateRinging;
+    } else if (mOutgoingState == OutgoingStateRinging) {
+        QCOMPARE(groupMembersAdded.size(), 1);
+        QVERIFY(groupLocalPendingMembersAdded.isEmpty());
+        QVERIFY(groupRemotePendingMembersAdded.isEmpty());
+        QVERIFY(groupMembersRemoved.isEmpty());
+
+        QCOMPARE(mChan->groupContacts().size(), 2);
+        QVERIFY(mChan->groupContacts().contains(otherContact));
+        QCOMPARE(mChan->awaitingRemoteAnswer(), false);
+
+        qDebug() << "call now answered";
+
+        mOutgoingState = OutgoingStateDone;
+        mOutgoingAudioDone = true;
+
+        // Exit if we already got finished() from requestStreams() - otherwise the finish callback
+        // will exit
+        if (mOutgoingGotRequestStreamsFinished) {
+            mLoop->exit(0);
+        }
+    }
+
+    qDebug() << "group members changed";
+    mChangedCurrent = groupMembersAdded;
+    mChangedLP = groupLocalPendingMembersAdded;
+    mChangedRP = groupRemotePendingMembersAdded;
+    mChangedRemoved = groupMembersRemoved;
+    mDetails = details;
+}
+
 void TestStreamedMediaChan::onGroupMembersChanged(
         const Contacts &groupMembersAdded,
         const Contacts &groupLocalPendingMembersAdded,
@@ -216,10 +395,111 @@ void TestStreamedMediaChan::onChanInvalidated(Tp::DBusProxy *proxy,
     mLoop->exit(0);
 }
 
+void TestStreamedMediaChan::expectTerminateRequestStreamsFinished(PendingOperation *op)
+{
+    QVERIFY(op->isFinished());
+
+    if (op->isError()) {
+        // FIXME: should the error be something else, actually? Such as, perchance,
+        // org.freedesktop.Telepathy.Error.Terminated? (fd.o #29757).
+        QCOMPARE(op->errorName(), QLatin1String("org.freedesktop.Telepathy.Error.Cancelled"));
+        qDebug() << "The remote hung up before we even got to take a look at the stream!";
+        mTerminateState = TerminateStateTerminated;
+        mLoop->exit(0);
+        return;
+    }
+
+    QVERIFY(op->isValid());
+
+    PendingMediaStreams *pms = qobject_cast<PendingMediaStreams*>(op);
+    mRequestStreamsReturn = pms->streams();
+
+    qDebug() << "stream requested successfully";
+
+    // Only advance to Requested if the remote moving to RP hasn't already advanced it to Ringing or
+    // even Answered - tbf it seems the StreamedMediaChannel semantics are quite hard for
+    // application code to get right because the events can happen in whichever order. Should this
+    // be considered a bug by itself? It'd probably be pretty hard to fix so I hope not :D
+    if (mTerminateState == TerminateStateInitial) {
+        mTerminateState = TerminateStateRequested;
+    }
+}
+
+void TestStreamedMediaChan::onTerminateGroupMembersChanged(
+        const Contacts &groupMembersAdded,
+        const Contacts &groupLocalPendingMembersAdded,
+        const Contacts &groupRemotePendingMembersAdded,
+        const Contacts &groupMembersRemoved,
+        const Channel::GroupMemberChangeDetails &details)
+{
+    // At this point, mRequestContactsReturn should still contain the contact we requested the
+    // stream for
+    ContactPtr otherContact = mRequestContactsReturn.first();
+
+    if (mTerminateState == TerminateStateInitial || mTerminateState == TerminateStateRequested) {
+        // The target should have become remote pending now
+        QVERIFY(groupMembersAdded.isEmpty());
+        QVERIFY(groupLocalPendingMembersAdded.isEmpty());
+        QCOMPARE(groupRemotePendingMembersAdded.size(), 1);
+        QVERIFY(groupMembersRemoved.isEmpty());
+
+        QVERIFY(mChan->groupRemotePendingContacts().contains(otherContact));
+        QCOMPARE(mChan->awaitingRemoteAnswer(), true);
+
+        qDebug() << "call now ringing";
+
+        mTerminateState = TerminateStateRinging;
+    } else if (mTerminateState == TerminateStateRinging) {
+        QCOMPARE(groupMembersAdded.size(), 1);
+        QVERIFY(groupLocalPendingMembersAdded.isEmpty());
+        QVERIFY(groupRemotePendingMembersAdded.isEmpty());
+        QVERIFY(groupMembersRemoved.isEmpty());
+
+        QCOMPARE(mChan->groupContacts().size(), 2);
+        QVERIFY(mChan->groupContacts().contains(otherContact));
+        QCOMPARE(mChan->awaitingRemoteAnswer(), false);
+
+        qDebug() << "call now answered";
+
+        mTerminateState = TerminateStateAnswered;
+    } else if (mTerminateState == TerminateStateAnswered) {
+        // It might be actually that currently this won't happen before invalidated() is emitted, so
+        // we'll never reach this due to having exited the mainloop already - but it's entirely
+        // valid for the library to signal either the invalidation or removing the members first, so
+        // let's verify the member change in case it does that first.
+
+        qDebug() << "membersChanged() after the call was answered - the remote probably hung up";
+
+        QVERIFY(groupMembersAdded.isEmpty());
+        QVERIFY(groupLocalPendingMembersAdded.isEmpty());
+        QVERIFY(groupRemotePendingMembersAdded.isEmpty());
+        QVERIFY(groupMembersRemoved.contains(otherContact) ||
+                groupMembersRemoved.contains(mChan->groupSelfContact())); // can be either, or both
+
+        // the invalidated handler will change state due to the fact that we might get 0-2 of these,
+        // but always exactly one invalidated()
+    }
+
+    qDebug() << "group members changed";
+    mChangedCurrent = groupMembersAdded;
+    mChangedLP = groupLocalPendingMembersAdded;
+    mChangedRP = groupRemotePendingMembersAdded;
+    mChangedRemoved = groupMembersRemoved;
+    mDetails = details;
+}
+
+void TestStreamedMediaChan::onTerminateChanInvalidated(Tp::DBusProxy *proxy,
+        const QString &errorName, const QString &errorMessage)
+{
+    qDebug() << "chan invalidated:" << errorName << "-" << errorMessage;
+    mTerminateState = TerminateStateTerminated;
+    mLoop->exit(0);
+}
+
 void TestStreamedMediaChan::onNewChannels(const Tp::ChannelDetailsList &channels)
 {
     qDebug() << "new channels";
-    foreach (const Tp::ChannelDetails &details, channels) {
+    Q_FOREACH (const Tp::ChannelDetails &details, channels) {
         QString channelType = details.properties.value(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".ChannelType")).toString();
         bool requested = details.properties.value(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".Requested")).toBool();
         qDebug() << " channelType:" << channelType;
@@ -238,6 +518,7 @@ void TestStreamedMediaChan::onNewChannels(const Tp::ChannelDetailsList &channels
 void TestStreamedMediaChan::onLocalHoldStateChanged(Tp::LocalHoldState localHoldState,
         Tp::LocalHoldStateReason localHoldStateReason)
 {
+    qDebug() << "local hold state changed:" << localHoldState << localHoldStateReason;
     mLocalHoldStates.append(localHoldState);
     mLocalHoldStateReasons.append(localHoldStateReason);
     mLoop->exit(0);
@@ -260,7 +541,8 @@ void TestStreamedMediaChan::initTestCase()
             EXAMPLE_TYPE_CALLABLE_CONNECTION,
             "account", "me@example.com",
             "protocol", "example",
-            0));
+            "simulation-delay", 1,
+            NULL));
     QVERIFY(mConnService != 0);
     QVERIFY(tp_base_connection_register(TP_BASE_CONNECTION(mConnService),
                 "example", &name, &connPath, &error));
@@ -364,36 +646,39 @@ void TestStreamedMediaChan::testOutgoingCall()
     QCOMPARE(mLoop->exec(), 2);
     QCOMPARE(mRequestStreamsReturn.size(), 0);
 
-    // Request audio stream
+    // Request audio stream, and wait for:
+    //  - the request to finish
+    //  - the contact to appear on RP
+    //  - the contact to accept the call
+    mOutgoingState = OutgoingStateInitial;
+    mOutgoingAudioDone = false;
+    mOutgoingGotRequestStreamsFinished = false;
+
+    QVERIFY(connect(mChan.data(),
+                    SIGNAL(groupMembersChanged(
+                            const Tp::Contacts &,
+                            const Tp::Contacts &,
+                            const Tp::Contacts &,
+                            const Tp::Contacts &,
+                            const Tp::Channel::GroupMemberChangeDetails &)),
+                    SLOT(onOutgoingGroupMembersChanged(
+                            const Tp::Contacts &,
+                            const Tp::Contacts &,
+                            const Tp::Contacts &,
+                            const Tp::Contacts &,
+                            const Tp::Channel::GroupMemberChangeDetails &))));
+
+    qDebug() << "requesting audio stream";
+
     QVERIFY(connect(mChan->requestStreams(otherContact,
                         QList<Tp::MediaStreamType>() << Tp::MediaStreamTypeAudio),
                     SIGNAL(finished(Tp::PendingOperation*)),
-                    SLOT(expectRequestStreamsFinished(Tp::PendingOperation*))));
+                    SLOT(expectOutgoingRequestStreamsFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
-    QCOMPARE(mRequestStreamsReturn.size(), 1);
-    MediaStreamPtr stream = mRequestStreamsReturn.first();
-    QCOMPARE(stream->contact(), otherContact);
-    QCOMPARE(stream->type(), Tp::MediaStreamTypeAudio);
-    QCOMPARE(stream->state(), Tp::MediaStreamStateDisconnected);
-    QCOMPARE(stream->direction(), Tp::MediaStreamDirectionBidirectional);
+    QCOMPARE(static_cast<int>(mOutgoingState), static_cast<int>(OutgoingStateDone));
+    QCOMPARE(mOutgoingAudioDone, true);
 
-    QCOMPARE(mChan->streams().size(), 1);
-    QVERIFY(mChan->streams().contains(stream));
-
-    // wait the contact to appear on RP
-    while (mChan->groupRemotePendingContacts().size() == 0) {
-        mLoop->processEvents();
-    }
-    QVERIFY(mChan->groupRemotePendingContacts().contains(otherContact));
-    QCOMPARE(mChan->awaitingRemoteAnswer(), true);
-
-    // wait the contact to accept the call
-    while (mChan->groupContacts().size() != 2) {
-        mLoop->processEvents();
-    }
-    QCOMPARE(mChan->groupContacts().size(), 2);
-    QCOMPARE(mChan->awaitingRemoteAnswer(), false);
-    QVERIFY(mChan->groupContacts().contains(otherContact));
+    qDebug() << "requesting video stream";
 
     // Request video stream
     QVERIFY(connect(mChan->requestStream(otherContact, Tp::MediaStreamTypeVideo),
@@ -401,11 +686,15 @@ void TestStreamedMediaChan::testOutgoingCall()
                     SLOT(expectRequestStreamsFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mRequestStreamsReturn.size(), 1);
-    stream = mRequestStreamsReturn.first();
+    MediaStreamPtr stream = mRequestStreamsReturn.first();
     QCOMPARE(stream->contact(), otherContact);
     QCOMPARE(stream->type(), Tp::MediaStreamTypeVideo);
-    QCOMPARE(stream->state(), Tp::MediaStreamStateDisconnected);
-    QCOMPARE(stream->direction(), Tp::MediaStreamDirectionBidirectional);
+
+    // These checks can't work reliably, unless we add some complex backdoors to the test service,
+    // to only start changing state / direction when we explicitly tell it so (not automatically
+    // when we have requested the stream)
+    // QCOMPARE(stream->state(), Tp::MediaStreamStateDisconnected);
+    // QCOMPARE(stream->direction(), Tp::MediaStreamDirectionBidirectional);
 
     QCOMPARE(mChan->streams().size(), 2);
     QVERIFY(mChan->streams().contains(stream));
@@ -417,6 +706,8 @@ void TestStreamedMediaChan::testOutgoingCall()
     stream = mChan->streamsForType(Tp::MediaStreamTypeAudio).first();
     QVERIFY(stream);
 
+    qDebug() << "removing audio stream";
+
     QVERIFY(connect(mChan.data(),
                     SIGNAL(streamRemoved(const Tp::MediaStreamPtr &)),
                     SLOT(onStreamRemoved(const Tp::MediaStreamPtr &))));
@@ -425,6 +716,7 @@ void TestStreamedMediaChan::testOutgoingCall()
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     if (mChan->streams().size() == 2) {
+        qDebug() << "re-entering mainloop to wait for stream removal being signaled";
         // wait stream removed signal
         QCOMPARE(mLoop->exec(), 0);
     }
@@ -437,6 +729,23 @@ void TestStreamedMediaChan::testOutgoingCall()
     stream = mChan->streamsForType(Tp::MediaStreamTypeVideo).first();
     QVERIFY(stream);
 
+    qDebug() << "changing stream direction, currently" << stream->direction();
+    qDebug() << "state currently" << stream->state();
+
+    if (stream->state() != Tp::MediaStreamStateConnected) {
+        QVERIFY(connect(mChan.data(),
+                    SIGNAL(streamStateChanged(const Tp::MediaStreamPtr &,
+                            Tp::MediaStreamState)),
+                    SLOT(onStreamStateChanged(const Tp::MediaStreamPtr &,
+                            Tp::MediaStreamState))));
+    } else {
+        // Pretend that we saw the SSC to Connected (although it might have happened even before the
+        // stream request finished, in which case we have no change of catching it, because we don't
+        // have the stream yet)
+        mSSCStreamReturn = stream;
+        mSSCStateReturn = Tp::MediaStreamStateConnected;
+    }
+
     QVERIFY(connect(mChan.data(),
                     SIGNAL(streamDirectionChanged(const Tp::MediaStreamPtr &,
                                                   Tp::MediaStreamDirection,
@@ -444,16 +753,12 @@ void TestStreamedMediaChan::testOutgoingCall()
                     SLOT(onStreamDirectionChanged(const Tp::MediaStreamPtr &,
                                                   Tp::MediaStreamDirection,
                                                   Tp::MediaStreamPendingSend))));
-    QVERIFY(connect(mChan.data(),
-                    SIGNAL(streamStateChanged(const Tp::MediaStreamPtr &,
-                                              Tp::MediaStreamState)),
-                    SLOT(onStreamStateChanged(const Tp::MediaStreamPtr &,
-                                              Tp::MediaStreamState))));
     QVERIFY(connect(stream->requestDirection(Tp::MediaStreamDirectionReceive),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     while (!mSDCStreamReturn || !mSSCStreamReturn) {
+        qDebug() << "re-entering mainloop to wait for stream direction change and state change";
         // wait direction and state changed signal
         QCOMPARE(mLoop->exec(), 0);
     }
@@ -507,15 +812,23 @@ void TestStreamedMediaChan::testOutgoingCallBusy()
     QVERIFY(connect(mChan->requestStreams(otherContact,
                         QList<Tp::MediaStreamType>() << Tp::MediaStreamTypeAudio),
                     SIGNAL(finished(Tp::PendingOperation*)),
-                    SLOT(expectRequestStreamsFinished(Tp::PendingOperation*))));
+                    SLOT(expectBusyRequestStreamsFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
 
-    QVERIFY(connect(mChan.data(),
+    if (mChan->isValid()) {
+        qDebug() << "waiting for the channel to become invalidated";
+
+        QVERIFY(connect(mChan.data(),
                     SIGNAL(invalidated(Tp::DBusProxy *,
-                                       const QString &, const QString &)),
+                            const QString &, const QString &)),
                     SLOT(onChanInvalidated(Tp::DBusProxy *,
-                                           const QString &, const QString &))));
-    QCOMPARE(mLoop->exec(), 0);
+                            const QString &, const QString &))));
+        QCOMPARE(mLoop->exec(), 0);
+    } else {
+        qDebug() << "not waiting for the channel to become invalidated, it has been invalidated" <<
+            "already";
+    }
+
     QCOMPARE(mChan->groupContacts().size(), 0);
     QCOMPARE(mChan->groupLocalPendingContacts().size(), 0);
     QCOMPARE(mChan->groupRemotePendingContacts().size(), 0);
@@ -644,6 +957,23 @@ void TestStreamedMediaChan::testOutgoingCallTerminate()
     QCOMPARE(mChan->awaitingLocalAnswer(), false);
     QVERIFY(mChan->groupContacts().contains(mConn->selfContact()));
 
+    // Request audio stream, and verify that following doing so, we get events for:
+    //  [0-2].5) the stream request finishing (sadly this can happen before, or between, any of the
+    //           following) - is this a bug?
+    //  1) the remote appearing on the RP contacts -> should be awaitingRemoteAnswer()
+    //  2) the remote answering the call -> should not be awaitingRemoteAnswer(), should have us and
+    //     them as the current members
+    //  3) the channel being invalidated (due to the remote having terminated the call) -> exits the
+    //     mainloop
+    //
+    //  Previously, this test used to spin the mainloop until each of the events had seemingly
+    //  happened, only checking for the events between the iterations. This is race-prone however,
+    //  as multiple events can happen in one mainloop iteration if the test executes slowly compared
+    //  with the simulated network events from the service (eg. in valgrind or under high system
+    //  load).
+
+    mTerminateState = TerminateStateInitial;
+
     QVERIFY(connect(mChan.data(),
                     SIGNAL(groupMembersChanged(
                             const Tp::Contacts &,
@@ -651,42 +981,29 @@ void TestStreamedMediaChan::testOutgoingCallTerminate()
                             const Tp::Contacts &,
                             const Tp::Contacts &,
                             const Tp::Channel::GroupMemberChangeDetails &)),
-                    SLOT(onGroupMembersChanged(
+                    SLOT(onTerminateGroupMembersChanged(
                             const Tp::Contacts &,
                             const Tp::Contacts &,
                             const Tp::Contacts &,
                             const Tp::Contacts &,
                             const Tp::Channel::GroupMemberChangeDetails &))));
 
-    // Request audio stream
-    QVERIFY(connect(mChan->requestStreams(otherContact,
-                        QList<Tp::MediaStreamType>() << Tp::MediaStreamTypeAudio),
-                    SIGNAL(finished(Tp::PendingOperation*)),
-                    SLOT(expectRequestStreamsFinished(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
-
-    // wait the contact to appear on RP
-    while (mChan->groupRemotePendingContacts().size() == 0) {
-        mLoop->processEvents();
-    }
-    QVERIFY(mChan->groupRemotePendingContacts().contains(otherContact));
-    QCOMPARE(mChan->awaitingRemoteAnswer(), true);
-
-    // wait the contact to accept the call
-    while (mChan->groupContacts().size() != 2) {
-        mLoop->processEvents();
-    }
-    QCOMPARE(mChan->groupContacts().size(), 2);
-    QCOMPARE(mChan->awaitingRemoteAnswer(), false);
-    QVERIFY(mChan->groupContacts().contains(otherContact));
-
-    // wait the contact to terminate the call
     QVERIFY(connect(mChan.data(),
                     SIGNAL(invalidated(Tp::DBusProxy *,
                                        const QString &, const QString &)),
-                    SLOT(onChanInvalidated(Tp::DBusProxy *,
+                    SLOT(onTerminateChanInvalidated(Tp::DBusProxy *,
                                            const QString &, const QString &))));
+
+    qDebug() << "calling, hope somebody answers and doesn't immediately hang up!";
+
+    QVERIFY(connect(mChan->requestStreams(otherContact,
+                        QList<Tp::MediaStreamType>() << Tp::MediaStreamTypeAudio),
+                    SIGNAL(finished(Tp::PendingOperation*)),
+                    SLOT(expectTerminateRequestStreamsFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
+    QCOMPARE(static_cast<int>(mTerminateState), static_cast<int>(TerminateStateTerminated));
+
+    qDebug() << "oh crap, nobody wants to talk to me";
 }
 
 
@@ -734,6 +1051,8 @@ void TestStreamedMediaChan::testIncomingCall()
     QCOMPARE(stream->channel(), mChan);
     QCOMPARE(stream->type(), Tp::MediaStreamTypeAudio);
 
+    qDebug() << "requesting a stream with a bad type";
+
     // RequestStreams with bad type must fail
     QVERIFY(connect(mChan->requestStream(otherContact, (Tp::MediaStreamType) -1),
                     SIGNAL(finished(Tp::PendingOperation*)),
@@ -741,6 +1060,8 @@ void TestStreamedMediaChan::testIncomingCall()
     QCOMPARE(mLoop->exec(), 2);
     QCOMPARE(mRequestStreamsReturn.size(), 0);
 
+    qDebug() << "requesting a video stream";
+
     // Request video stream
     QVERIFY(connect(mChan->requestStream(otherContact, Tp::MediaStreamTypeVideo),
                     SIGNAL(finished(Tp::PendingOperation*)),
@@ -750,8 +1071,12 @@ void TestStreamedMediaChan::testIncomingCall()
     stream = mRequestStreamsReturn.first();
     QCOMPARE(stream->contact(), otherContact);
     QCOMPARE(stream->type(), Tp::MediaStreamTypeVideo);
-    QCOMPARE(stream->state(), Tp::MediaStreamStateDisconnected);
-    QCOMPARE(stream->direction(), Tp::MediaStreamDirectionBidirectional);
+
+    // These checks can't work reliably, unless we add some complex backdoors to the test service,
+    // to only start changing state / direction when we explicitly tell it so (not automatically
+    // when we have requested the stream)
+    // QCOMPARE(stream->state(), Tp::MediaStreamStateDisconnected);
+    // QCOMPARE(stream->direction(), Tp::MediaStreamDirectionBidirectional);
 
     QCOMPARE(mChan->streams().size(), 2);
     QVERIFY(mChan->streams().contains(stream));
@@ -763,6 +1088,8 @@ void TestStreamedMediaChan::testIncomingCall()
     stream = mChan->streamsForType(Tp::MediaStreamTypeAudio).first();
     QVERIFY(stream);
 
+    qDebug() << "removing the audio stream";
+
     QVERIFY(connect(mChan.data(),
                     SIGNAL(streamRemoved(const Tp::MediaStreamPtr &)),
                     SLOT(onStreamRemoved(const Tp::MediaStreamPtr &))));
@@ -783,6 +1110,23 @@ void TestStreamedMediaChan::testIncomingCall()
     stream = mChan->streamsForType(Tp::MediaStreamTypeVideo).first();
     QVERIFY(stream);
 
+    qDebug() << "requesting direction (false, true) - currently" << stream->direction();
+    qDebug() << "current stream state" << stream->state();
+
+    if (stream->state() != Tp::MediaStreamStateConnected) {
+        QVERIFY(connect(mChan.data(),
+                    SIGNAL(streamStateChanged(const Tp::MediaStreamPtr &,
+                            Tp::MediaStreamState)),
+                    SLOT(onStreamStateChanged(const Tp::MediaStreamPtr &,
+                            Tp::MediaStreamState))));
+    } else {
+        // Pretend that we saw the SSC to Connected (although it might have happened even before the
+        // stream request finished, in which case we have no change of catching it, because we don't
+        // have the stream yet)
+        mSSCStreamReturn = stream;
+        mSSCStateReturn = Tp::MediaStreamStateConnected;
+    }
+
     QVERIFY(connect(mChan.data(),
                     SIGNAL(streamDirectionChanged(const Tp::MediaStreamPtr &,
                                                   Tp::MediaStreamDirection,
@@ -790,17 +1134,13 @@ void TestStreamedMediaChan::testIncomingCall()
                     SLOT(onStreamDirectionChanged(const Tp::MediaStreamPtr &,
                                                   Tp::MediaStreamDirection,
                                                   Tp::MediaStreamPendingSend))));
-    QVERIFY(connect(mChan.data(),
-                    SIGNAL(streamStateChanged(const Tp::MediaStreamPtr &,
-                                              Tp::MediaStreamState)),
-                    SLOT(onStreamStateChanged(const Tp::MediaStreamPtr &,
-                                              Tp::MediaStreamState))));
     QVERIFY(connect(stream->requestDirection(false, true),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     while (!mSDCStreamReturn || !mSSCStreamReturn) {
         // wait direction and state changed signal
+        qDebug() << "re-entering mainloop to wait for stream direction change and state change";
         QCOMPARE(mLoop->exec(), 0);
     }
     QCOMPARE(mSDCStreamReturn, stream);
@@ -916,13 +1256,11 @@ void TestStreamedMediaChan::testHoldNoUnhold()
                     SIGNAL(localHoldStateChanged(Tp::LocalHoldState, Tp::LocalHoldStateReason)),
                     SLOT(onLocalHoldStateChanged(Tp::LocalHoldState, Tp::LocalHoldStateReason))));
     // Request hold
-    QVERIFY(connect(mChan->requestHold(true),
-                    SIGNAL(finished(Tp::PendingOperation*)),
-                    SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
-    while (mLocalHoldStates.size() != 2) {
-        QCOMPARE(mLoop->exec(), 0);
+    QPointer<PendingOperation> holdOp = mChan->requestHold(true);
+    while (mLocalHoldStates.size() != 2 || (holdOp && !holdOp->isFinished())) {
+        mLoop->processEvents();
     }
+    QCOMPARE(!holdOp || holdOp->isValid(), true);
     QCOMPARE(mLocalHoldStates.first(), static_cast<uint>(LocalHoldStatePendingHold));
     QCOMPARE(mLocalHoldStateReasons.first(), static_cast<uint>(LocalHoldStateReasonRequested));
     QCOMPARE(mLocalHoldStates.last(), static_cast<uint>(LocalHoldStateHeld));
@@ -933,6 +1271,8 @@ void TestStreamedMediaChan::testHoldNoUnhold()
     mLocalHoldStates.clear();
     mLocalHoldStateReasons.clear();
 
+    qDebug() << "requesting failing unhold";
+
     // Request unhold (fail)
     QVERIFY(connect(mChan->requestHold(false),
                     SIGNAL(finished(Tp::PendingOperation*)),
@@ -1172,6 +1512,16 @@ void TestStreamedMediaChan::testDTMFNoContinuousTone()
 void TestStreamedMediaChan::cleanup()
 {
     mChan.reset();
+    mRequestContactsReturn.clear();
+    mRequestStreamsReturn.clear();
+    mStreamRemovedReturn.reset();
+    mChangedCurrent.clear();
+    mChangedLP.clear();
+    mChangedRP.clear();
+    mChangedRemoved.clear();
+    mDetails = Channel::GroupMemberChangeDetails();
+    mSDCStreamReturn.reset();
+    mSSCStreamReturn.reset();
     cleanupImpl();
 }
 
@@ -1191,6 +1541,15 @@ void TestStreamedMediaChan::cleanupTestCase()
                             SLOT(quit())));
             QCOMPARE(mLoop->exec(), 0);
         }
+
+        processDBusQueue(mConn.data());
+
+        mConn.reset();
+    }
+
+    if (mConnService) {
+        g_object_unref(mConnService);
+        mConnService = 0;
     }
 
     cleanupTestCaseImpl();
