@@ -39,6 +39,7 @@ enum
 {
   PROP_OBJECT_PATH = 1,
   PROP_CONNECTION,
+  PROP_INTERFACES,
   PROP_HANDLE,
   PROP_SIMULATION_DELAY,
   PROP_LOCALLY_REQUESTED,
@@ -119,6 +120,8 @@ constructed (GObject *object)
     }
 }
 
+static const gchar * const empty_strv[] = { NULL };
+
 static void
 get_property (GObject *object,
     guint property_id,
@@ -133,6 +136,10 @@ get_property (GObject *object,
       g_value_set_string (value, self->priv->object_path);
       break;
 
+    case PROP_INTERFACES:
+      g_value_set_static_boxed (value, empty_strv);
+      break;
+
     case PROP_HANDLE:
       g_value_set_uint (value, self->priv->handle);
       break;
@@ -251,6 +258,7 @@ example_call_stream_class_init (ExampleCallStreamClass *klass)
 {
   static TpDBusPropertiesMixinPropImpl stream_props[] = {
       { "Senders", "senders", NULL },
+      { "Interfaces", "interfaces", NULL },
       { NULL }
   };
   static TpDBusPropertiesMixinIfaceImpl prop_interfaces[] = {
@@ -310,6 +318,11 @@ example_call_stream_class_init (ExampleCallStreamClass *klass)
       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_SENDERS, param_spec);
 
+  param_spec = g_param_spec_boxed ("interfaces", "Interfaces",
+      "List of D-Bus interfaces",
+      G_TYPE_STRV, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_INTERFACES, param_spec);
+
   signals[SIGNAL_REMOVED] = g_signal_new ("removed",
       G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST, 0, NULL, NULL,
       g_cclosure_marshal_VOID__VOID,
